
file_id = "ADA_LED8X8_DISP"

--[[

●機能概要

Adafruit Mini 8x8 LED Matrix w/I2C Backpackにピクセルデータを表示する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

data		LED 8x8マトリックスに表示する点灯パターン
			8 バイト分のデータを16進数文字列で連結したものを指定する
			MSB は LED マトリックスの ROW#0 側で LSB が ROW#7 に対応する。

			全ピクセル点灯									"FFFFFFFFFFFFFFFF"
			全ピクセル消灯									"0000000000000000"
			四角枠のピクセルを点灯							"FF818181818181FF"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_WRITE_COMMAND]をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["data"]) then
	log_msg("parameter error",file_id)
	error()
end

local param = {}
param["com"] = g_params["com"]
param["slave_addr"] = "70"

---------------------------------------------------------------------------------
-- LED マトリックスデバイスの初期化
---------------------------------------------------------------------------------
function init_device()
	-----------------------------
	-- turn on oscillator
	-----------------------------
	param["data"] = "21"
	if not script_exec("ARDUINO/I2C_WRITE",param) then error() end

	-----------------------------
	-- blink off and display on
	-----------------------------
	param["data"] = "81"
	if not script_exec("ARDUINO/I2C_WRITE",param) then error() end

	-----------------------------
	--  brightness	
	-----------------------------
--	param["data"] = "EF" -- max
	param["data"] = "E3" -- 1/4
	if not script_exec("ARDUINO/I2C_WRITE",param) then error() end

	log_msg("device initialized",file_id)
end

---------------------------------------------------------
-- カラムデータバイトの低位ビットと高位ビットを入れ替える
---------------------------------------------------------
function swap_bits(x)
	local ret = 0
	if (bit_and(x,0x01) ~= 0) then ret = ret + 0x80 end
	if (bit_and(x,0x02) ~= 0) then ret = ret + 0x40 end
	if (bit_and(x,0x04) ~= 0) then ret = ret + 0x20 end
	if (bit_and(x,0x08) ~= 0) then ret = ret + 0x10 end
	if (bit_and(x,0x10) ~= 0) then ret = ret + 0x08 end
	if (bit_and(x,0x20) ~= 0) then ret = ret + 0x04 end
	if (bit_and(x,0x40) ~= 0) then ret = ret + 0x02 end
	if (bit_and(x,0x80) ~= 0) then ret = ret + 0x01 end
	return ret
end

--------------------------------------------------
-- カラムデータバイトを 1 ビット右ローテートさせる
--------------------------------------------------
function wrap_pixel(x)
	local ret = bit_rshift(x,1)
	if (bit_and(x,0x01) ~= 0) then
		ret = ret + 0x80
	end 
	return ret
end

---------------------------------------------
-- 入力データを LED マトリックス用に変換する
---------------------------------------------
local data16 = ""
local arr = hex_to_tbl(g_params["data"])
for key,val in ipairs(arr) do
	data16 = data16 .. bit_tohex(wrap_pixel(swap_bits(val)),2) .. "00"
end

---------------------------------------------------------------
-- このスクリプトが最初に呼ばれたときにはデバイスを初期設定する
---------------------------------------------------------------
local stat,val = get_shared_data("Adafruit_LED8X8_" .. g_params["com"])
if not stat then error() end
if (val == "") then
	init_device()
	if not set_shared_data("Adafruit_LED8X8_" .. g_params["com"],"1") then error() end
end

---------------------------------------------------------------
-- レジスタ 0x00 からピクセルデータを順に書き込む
---------------------------------------------------------------
param["data"] = "00" .. data16
if not script_exec("ARDUINO/I2C_WRITE",param) then error() end

