
file_id = "I2C_RTC_GET"

--[[

●機能概要

I2C バスに接続した RTCデバイス(RTC-8564NB) の時刻を取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
RTC				yyyy/mm/dd HH:MM:SS 形式の時刻
				RTC に時刻が設定されていない場合には、
				"時刻が設定されていません" の文字列が返る		"2014/01/01 12:33:22"

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_WRITE_AND_READ_QUERY] [I2C_READ_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"]) then
	log_msg("parameter error",file_id)
	error()
end


---------------------------------------------------------------
-- RTCデバイスのアドレス 0x00 からレジスタ値を9 バイト読み込む
---------------------------------------------------------------

local param = {}
param["com"] = g_params["com"]
param["slave_addr"] = "51"
param["count"] = "9"
param["data"] = "00";

local stat,result = script_exec2("ARDUINO/I2C_WRITE_READ",param)
if not stat then error() end


---------------------------------------------------------
-- レジスタ値の有効データ部分のみをマスクして結果を表示
---------------------------------------------------------
local reg = hex_to_tbl(result["I2C_DATA"])

local msg;
if bit_and(reg[3],0x80) > 0 then
	msg = "時刻が設定されていません"
else
	msg = 	string.format("20%2.2x/%2.2x/%2.2x %2.2x:%2.2x:%2.2x",
			reg[9],
			bit_and(reg[8],0x1f),
			bit_and(reg[6],0x3f),
			bit_and(reg[5],0x3f),
			bit_and(reg[4],0x7f),
			bit_and(reg[3],0x7f))
end;

log_msg("RTC = " .. msg,file_id)

script_result(g_taskid,"RTC",msg)

