
file_id = "I2C_RTC_SET"

--[[

●機能概要

I2C バスに接続した RTCデバイス(RTC-8564NB) の初期設定と
RTC の時刻を DeviceServer の動作する PC の時計に合わせる

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_WRITE_COMMAND] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"]) then
	log_msg("parameter error",file_id)
	error()
end


---------------------------------------------------------------
-- レジスタ設定値 レジスタ 00 から時刻データを書き込む
---------------------------------------------------------------
local now = os.date "*t";
local param = {}
param["com"] = g_params["com"]
param["slave_addr"] = "51"
param["data"] = "00" .. string.format("0000%2.2d%2.2d%2.2d%2.2d%2.2d%2.2d%2.2d",
	now["sec"],now["min"],now["hour"],now["day"],now["wday"] - 1,now["month"],now["year"] - 2000);

local stat = script_exec("ARDUINO/I2C_WRITE",param)
if not stat then error() end


