
file_id = "SB16SEG4"

--[[

●機能概要

SB_16SEG4 ファームウエアを搭載した 16セグメントLED (4桁)に文字列を表示する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com			Arduino が接続された COM ポート名またはタイトル名		"COM4"

data		16セグメントLED に表示する文字列データ						"abcd"

clear		このパラメータを指定すると最初にLCD 表示内容をクリアする
			値は何でも構わない											"1"

slave_addr	I2C スレーブデバイスアドレス(16進数2桁)						"33"
			省略時は "33" (0x33) が指定される

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_WRITE_COMMAND]をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2015/07/02	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not g_params["com"] then
	log_msg("parameter error",file_id)
	error()
end

local param = {}
param["com"] = g_params["com"]
if g_params["slave_addr"] then
	param["slave_addr"] = g_params["slave_addr"]
else
	param["slave_addr"] = "33"
end

------------------------------------
-- 全てのセグメントを消灯
------------------------------------
if g_params["clear"] then
	param["data"] = "12"
	if not script_exec("ARDUINO/I2C_WRITE",param) then error() end
end

-----------------------------------------------------------------
-- SB_16SEG4 デバイスに文字列書き込み
-----------------------------------------------------------------
if g_params["data"] and (g_params["data"] ~= "") then
	local data = ""
	local hexarr = str_to_tbl(string.sub(g_params["data"],1,4))
	for key,val in ipairs(hexarr) do
		data = data .. bit_tohex(val,2)
	end
	param["data"] = "11" .. data
	if not script_exec("ARDUINO/I2C_WRITE",param) then error() end
end

