
file_id = "ARDUINO/I2C_WRITE"

--[[

●機能概要

Arduino I2C バスに接続したスレーブデバイスにデータを書き込む

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

slave_addr		I2C スレーブデバイスアドレス(16進数2桁)					"50"

data			スレーブデバイスに書き込むデータ(16進数)列				"0A0B0C"
				最大31バイトまで

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[I2C_WRITE_COMMAND] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

書き込みデータ数については、Arduino 側で動作させる FIRMATA スケッチプログラムの
仕様を参照してください。ここで指定した最大数以外にI2C スレーブデバイス毎の仕様に
よって制限がある場合がありますので注意してください。

●変更履歴

2014/04/20	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["slave_addr"] and g_params["data"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Firmata I2C_WRITE_COMMAND リクエストフレーム作成
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "67"
local slave_addr_hexstr = g_params["slave_addr"]
local data_hexstr = tbl_to_hex72(hex_to_tbl(g_params["data"]),2)
local send_data = sysex_start ..  firmata_request .. slave_addr_hexstr .. data_hexstr .. sysex_end

----------------------------------------------------------------------
-- リクエストフレーム送信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data)
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

