
file_id = "ARDUINO/VERSION"

--[[

●機能概要

Arduinoで動作している Firmataライブラリを使用したプログラムのバージョン番号を
取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名			"COM4"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            						値の例
---------------------------------------------------------------------------------

FIRMWARE_NAME	ファームウエア名							"SensorControlModule"

MAJOR_VERSION	ファームメジャーバージョン番号				"1"

MINOR_VERSION	ファームメジャーバージョン番号				"0"

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA ライブラリを
リンクしたスケッチプログラムを動作させてください。

●変更履歴

2014/03/04	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"]) then
	log_msg("parameter error",file_id)
	error()
end

local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "79"
local firmata_reply = "79"
local send_data = sysex_start .. firmata_request .. sysex_end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)
if not stat then error() end

---------------------------------------------------------------
-- REPORT_FIRMWARE パケット処理
---------------------------------------------------------------
local rec = hex_to_tbl(rdata)
local major = tostring(rec[3])
local minor = tostring(rec[4])
local stat2,firm_name = firmata_str_decode(string.sub(rdata,9,-3))

if not script_result(g_taskid,"FIRMWARE_NAME",firm_name) then error() end
if not script_result(g_taskid,"MAJOR_VERSION",major) then error() end
if not script_result(g_taskid,"MINOR_VERSION",minor) then error() end

