
--[[

●機能概要

FASTDB データベースに保存されているデータを検索して JSON配列で取得する。
検索パラメータには検索範囲日時や取得最大レコード数を指定できる。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

KeyList			検索対象とする FASTDB データベース中のキー名リスト
				複数指定するときにはカンマで区切る			
											"SENSOR_IR_Device4,SENSOR_IR_Node1"

TargetDateFrom	検索対象開始日付(YYYY/MM/DD)							"2010/01/31"
				TargetDateFrom, TargetTimeFrom 両方のパラメータ省略時には、
				検索対象に指定したキーで、最も過去に登録データから検索対象になる。
				（fastdb_search() ライブラリコール時に datetime_from パラメータを省略する）

				また、TargetDateFrom を省略して TargetTimeFrom のみを指定した
				場合にはスクリプト実行日が TargetDateFrom に設定される。


TargetTimeFrom	検索対象開始時刻(HH:MM:SS)							"13:25:0"
				TargetDateFrom, TargetTimeFrom 両方のパラメータ省略時には、
				検索対象に指定したキーで、最も過去に登録データから検索対象になる。
				（fastdb_search() ライブラリコール時に datetime_from パラメータを省略する）

				また、TargetDateFrom を省略して TargetTimeFrom のみを指定した
				場合にはスクリプト実行日が TargetDateFrom に設定される。

				TargetDateFrom を指定して、TargetTime パラメータのみを省略した場合には
				"0:0:0" が設定される

TargetDateUntil	検索対象終了日付(YYYY/MM/DD)							"2010/01/31"
				TargetDateUntil, TargetTimeUntil 両方のパラメータ省略時には、
				検索対象となったキーで、最も最近に登録したデータまでを検索する。
				（fastdb_search() ライブラリコール時に datetime_until パラメータを省略する）

				また、TargetDateUntil を省略して TargetTimeUntil のみを指定した
				場合にはスクリプト実行日が TargetDateUntil に設定される。


TargetTimeUntil	検索対象終了時刻(HH:MM:SS)							"13:25:0"
				TargetDateUntil, TargetTimeUntil 両方のパラメータ省略時には、
				検索対象となったキーで、最も最近に登録したデータまでを検索する。
				（fastdb_search() ライブラリコール時に datetime_until パラメータを省略する）

				また、TargetDateUntil を省略して TargetTimeUntil のみを指定した
				場合にはスクリプト実行日が TargetDateUntil に設定される。

				TargetDateUntil を指定して、TargetTimeUntil パラメータのみを省略した場合には
				"0:0:0" が設定される。この場合 TargetDateUntil に指定した日付のデータは検索
				結果に含まれない点に注意。もしTargetDateUntil に指定した日付のデータを含みたい場合には
				次の日の日付を TargetDateUntil に指定する。

MaxList															"1000"
				各キー毎の検索結果データ数の最大値。
				検索対象期間に当てはまるデータが MaxList で指定した数よりも多い場合には
				MaxList 分のデータがリターン値に帰り、BreakList に "True" が返ります。
				パラメータ省略時は 500 が指定される。
				
Direction
				"ascend"
					データを検索するときにTargetDateFrom, TargetTimeFrom に指定した日時から、
					新しいタイムスタンプの向きにデータ検索を行う。

				"descend"
					データを検索するときにTargetDateUntil, TargetTimeUntil に指定した日時から、
					古いタイムスタンプの向きにデータ検索を行う。
				
				"auto"
					検索条件から自動で判断する。
					もし、検索開始日時が指定された場合には "ascend" で検索する。
					それ以外は全て "descend" で検索する。


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
SeriesList		集計結果を jqplot の データ配列パラメータで指定できるような
				フォーマットに変換した JSON 文字列。
				KeyList に指定したキーの数だけ集計結果が格納される。

				集計単位期間の開始時刻と集計値のペアからなる集計データ <item> が作成される
				複数の集計データを合わせて集計配列 <series> を構成する。
				KeyList に指定したキーごとに 集計配列 <series> が作成されて、それらを配列で
				まとめたものが SeriesList になる。

LabelList:		KeyList で指定された文字列を jqplot の series ラベル・オプションパラメータで
				指定できるようなフォーマットに変換した JSON 文字列。

				検索対象期間に当てはまるデータが MaxList で指定した数よりも多い場合には
				MaxList 分のデータがリターン値に返り、LabelList 中のキー文字列には
				" <-->" 文字列がアペンドされます。


●リターンパラメータフォーマット

SeriesList := [
	[<series#1>],
	[<series#2>],
	..
	[<series#n>]
]

series := [
	[<item#1],
	[<item#2],
	..
	[<item#n]
]

item := ["<FASTDB に登録されたデータレコードのタイムスタンプ>"], <データレコードの値>]

SeriesList例：(2つのキー文字列を KeyList に指定した場合)

[
	[
		["2014/07/02 00:00:00",10],
		["2014/07/02 00:10:00",13],
		["2014/07/02 00:20:00",13.5],
		..
		["2014/07/02 23:50:00",20]
	],
	[
		["2014/07/02 00:00:00",12],
		["2014/07/02 00:10:00",12],
		["2014/07/02 00:20:00",12],
		..
		["2014/07/02 23:50:00",12.4]
	]
]

LabelList := [
	{"label":"<key#1>[ <-->]"},
	{"label":"<key#2>[ <-->]"},
	..
	{"label":"<key#n>[ <-->]"}
]


●備考


Web API 経由でこのスクリプトを実行するときに、URL パラメータに noquote=1 を指定
して、リターンパラメータの値を直接 JSON オブジェクトとしてアクセスします。

JavaScriptから以下のURL をコールしてJSON リプライデータを受信します。

http://<hostname>:<port>/command/json/script?session=<session_token>&name=FASTDB%2FSEARCH_JSON&noquote=1

●変更履歴

2018/11/7 新規作成

abs_agent  copyright(c) 2014-2018 All Blue System

]]

----------------------------------------------------------------------
-- KeyList パラメータからキー文字列配列 keys 作成
----------------------------------------------------------------------
local keys
if g_params["KeyList"] then
    keys = csv_to_tbl(g_params["KeyList"])
else
	log_msg("parameter error",g_script)
	error()
end

local p_datetime_from = ""
local p_datetime_until = ""

----------------------------------------------------------------------
-- 検索結果最大レコード数パラメータ設定
----------------------------------------------------------------------
local p_maxlist = 500
if g_params["MaxList"] then
	p_maxlist = tonumber(g_params["MaxList"])
end

---------------------------------------------------------------------------------
-- TargetDateFrom,TargetTimeFrom,TargetDateUntil,TargetTimeUntil パラメータ処理
---------------------------------------------------------------------------------
local now = os.date "*t"

if g_params["TargetDateFrom"] and g_params["TargetTimeFrom"] then
    p_datetime_from = g_params["TargetDateFrom"] .. " " .. g_params["TargetTimeFrom"]
end
if g_params["TargetDateFrom"] and (not g_params["TargetTimeFrom"]) then
    p_datetime_from = g_params["TargetDateFrom"] .. " 0:0:0"
end
if (not g_params["TargetDateFrom"]) and g_params["TargetTimeFrom"] then
    p_datetime_from = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTimeFrom"]
end
if g_params["TargetDateUntil"] and g_params["TargetTimeUntil"] then
    p_datetime_until = g_params["TargetDateUntil"] .. " " .. g_params["TargetTimeUntil"]
end
if g_params["TargetDateUntil"] and (not g_params["TargetTimeUntil"]) then
    p_datetime_until = g_params["TargetDateUntil"] .. " 0:0:0"
end
if (not g_params["TargetDateUntil"]) and g_params["TargetTimeUntil"] and (not g_params["TargetDateFrom"]) then
    p_datetime_until = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTimeUntil"]
end
if g_params["TargetDateFrom"] and g_params["TargetTimeFrom"] and (not g_params["TargetDateUntil"]) and g_params["TargetTimeUntil"] then
    p_datetime_until = g_params["TargetDateFrom"] .. " " .. g_params["TargetTimeUntil"]
end
if (not g_params["TargetDateFrom"]) and (not g_params["TargetTimeFrom"]) and g_params["TargetDateUntil"] and g_params["TargetTimeUntil"] then
    p_datetime_from = ""
    p_datetime_until = g_params["TargetDateUntil"] .. " " .. g_params["TargetTimeUntil"]
end
if (not g_params["TargetDateFrom"]) and (not g_params["TargetTimeFrom"]) and (not g_params["TargetDateUntil"]) and g_params["TargetTimeUntil"] then
    p_datetime_from = ""
    p_datetime_until = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTimeUntil"]
end
if (not g_params["TargetDateFrom"]) and g_params["TargetTimeFrom"] and (not g_params["TargetDateUntil"]) and g_params["TargetTimeUntil"] then
    p_datetime_from = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTimeFrom"]
    p_datetime_until = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTimeUntil"]
end

----------------------------------------------------------------------
-- 検索順パラメータ設定
----------------------------------------------------------------------
local p_descend = true
if g_params["Direction"] then
	if g_params["Direction"] == "ascend" then
		p_descend = false
	end
	if g_params["Direction"] == "auto" then
		if p_datetime_from ~= "" then
			p_descend = false
		end
	end
end

----------------------------------------------------------------------
-- データ検索
----------------------------------------------------------------------
log_msg("keys:" .. g_params["KeyList"] .. " p_maxlist:" .. tostring(p_maxlist)
		.. " p_descend:" .. tostring(p_descend) .. " p_datetime_from:"
		.. p_datetime_from .. " p_datetime_until:" .. p_datetime_until,g_script)

local series_json = '['
local label_json = '['
local first_series = true
local stat,value_list,timestamp_list,break_flag
for k,v in ipairs(keys) do
	log_msg("searching .. " .. v,g_script)
	if first_series then
		first_series = false
	else
		series_json = series_json .. ","
		label_json = label_json .. ","
	end
	---------------------------
	-- キー(シリーズ)毎の検索
	---------------------------
	stat,value_list,timestamp_list,break_flag = fastdb_search(v,p_datetime_from,p_datetime_until,p_descend,p_maxlist)
	if not stat then error() end
	series_json = series_json .. '['
	if break_flag then
		label_json = label_json .. '{"label":"' .. v .. " <-->" .. '"}' 
	else
		label_json = label_json .. '{"label":"' .. v .. '"}' 
	end
	local first_item = true
	local arr_size = #timestamp_list
	if p_descend then -- SeriesList 配列を昇順(タイムスタンプが古いものから順)に揃える
		for key = arr_size,1,-1 do
			if first_item then
				first_item = false
			else
				series_json = series_json .. ","
			end
			series_json = series_json .. string.format('["%s",%g]',timestamp_list[key],value_list[key])
		end
	else
		for key = 1,arr_size,1 do
			if first_item then
				first_item = false
			else
				series_json = series_json .. ","
			end
			series_json = series_json .. string.format('["%s",%g]',timestamp_list[key],value_list[key])
		end

	end
	series_json = series_json .. ']'
end
series_json = series_json .. ']'
label_json = label_json .. ']'

--------------------------------------------
-- リターンパラメータに JSON 文字列を設定
--------------------------------------------
script_result("SeriesList",series_json)
script_result("LabelList",label_json)
