
--[[

●機能概要

FASTDB データベースに保存されているデータを集計して JSON配列で取得する。
集計パラメータには日、週、月の範囲を指定できる。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

KeyList			集計対象とする FASTDB データベース中のキー名リスト
				複数指定するときにはカンマで区切る			
											"SENSOR_IR_Device4,SENSOR_IR_Node1"

TargetDate		集計対象開始日付(YYYY/MM/DD)							"2010/01/31"
				パラメータ省略時には Range パラメータの指定によってデフォルト
				日付が設定される。
				Range パラメータを省略または "day" または "month" 指定時には、
				現在日が集計対象開始日になる。
				Rangeパラメターが "week","2d","month" の場合には現在日から
				その日数期間前の日付が集計対象開始日になる。

TargetTime		集計対象開始時刻(HH:MM:SS)								"13:25:0"
				TargetDate, TargetTime 両方のパラメータ省略時には、
				現在の日付・時刻(秒部分は切り捨てて xx:xx:00 になる)から集計期間分前の
				日付時刻が設定される。
				TargetDate を指定して、TargetTime パラメータのみを省略した場合には
				"0:0:0" が設定される
				Range パラメータに 1日以上の期間を指定した場合には、このパラメータ
				の指定は無視されて常に "0:0:0" からの集計期間になる

Range			集計期間を指定する。下記の値が指定可能でパラメータ省略時には
				"day" が選択される

				"5m"
					TargetDate, TargetTime に指定した日付時刻から 5分間を 2秒単位
					に集計する

				"hour"
					TargetDate, TargetTime に指定した日付時刻から 1時間を 30秒単位
					に集計する

				"6h"
					TargetDate, TargetTime に指定した日付時刻から 6時間を 1分単位
					に集計する

				"day" 
					TagetDate に指定した日付の 0:0:0 から 24:00:00 までの期間を
					5 分単位に集計する。

				"2d" 
					TagetDate に指定した日付の 0:0:0 から次の日の 24:00:00 までの期間を
					10 分単位に集計する

				"week" 
					TagetDate に指定した日付の 0:0:0 から 7日間を 1時間単位に集計する

				"2w" 
					TagetDate に指定した日付の 0:0:0 から 14日間を 2時間単位に集計する

				"month" 
					TagetDate に指定した日付の同月 1日の 0:0:0 から月の最終日の
					24:00:00 までの期間を 4時間単位に集計する

Summary			集計単位ごとの期間で集計計算したときに使用する値を指定する
				パラメター省略時には "mean" が選択される				"mean"
				"mean"			平均値
				"total"			合計値
				"cumulative"	累計値
				"first"			最初(最も古いタイムスタンプを持つ)のデータ値

Interval		Range パラメータの指定によって予め決められた集計単位時間を変更して
				このパラメータで指定された秒を使用する。
				秒数で指定する。										"3600"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
SeriesList		集計結果を jqplot の データ配列パラメータで指定できるような
				フォーマットに変換した JSON 文字列。
				KeyList に指定したキーの数だけ集計結果が格納される。

				集計単位期間の開始時刻と集計値のペアからなる集計データ <item> が作成される
				複数の集計データを合わせて集計配列 <series> を構成する。
				KeyList に指定したキーごとに 集計配列 <series> が作成されて、それらを配列で
				まとめたものが SeriesList になる。

LabelList:		KeyList で指定された文字列を jqplot の series ラベル・オプションパラメータで
				指定できるようなフォーマットに変換した JSON 文字列。

●リターンパラメータフォーマット

SeriesList := [
	[<series#1>],
	[<series#2>],
	..
	[<series#n>]
]

series := [
	[<item#1],
	[<item#2],
	..
	[<item#n]
]

item := ["<集計単位の開始時刻", <集計値>]

SeriesList例：(2つのキー文字列を KeyList に指定した場合)

[
	[
		["2014/07/02 00:00:00",10],
		["2014/07/02 00:10:00",13],
		["2014/07/02 00:20:00",13.5],
		..
		["2014/07/02 23:50:00",20]
	],
	[
		["2014/07/02 00:00:00",12],
		["2014/07/02 00:10:00",12],
		["2014/07/02 00:20:00",12],
		..
		["2014/07/02 23:50:00",12.4]
	]
]

LabelList := [
	{"label":"<key#1>"},
	{"label":"<key#2>"},
	..
	{"label":"<key#n>"}
]


●備考

集計単位ごとの期間内にFASTDBデータレコードが見つからなかった場合には、
その単位期間の集計結果レコードは SeriesList 中に入りません。

Web API 経由でこのスクリプトを実行するときに、URL パラメータに noquote=1 を指定
して、リターンパラメータの値を直接 JSON オブジェクトとしてアクセスします。

JavaScriptから以下のURL をコールしてJSON リプライデータを受信します。

http://<hostname>:<port>/command/json/script?session=<session_token>&name=FASTDB%2FSUMMARY_JSON&noquote=1

●変更履歴

2021/3/3    集計間隔に "2週間"を追加した

2020/11/3   Summary パラメータに集計期間内の最初のデータ値を取得する "first" オプションを追加した

2020/9/25   集計間隔に "5分間"を追加した

2020/4/1    Summary パラメータに累計値を計算する "cumulative" オプションを追加した

2018/11/03  LabelList JSON 中の label:xxxx を "label":xxxxx に修正。
			備考欄のURL間違いを修正

2017/04/23  ver1.0 DeviceServer用に配布していたスクリプトを abs_agent 用に移植

abs_agent  copyright(c) 2014-2021 All Blue System

]]

----------------------------------------------------------------------
-- KeyList パラメータからキー文字列配列 keys 作成
----------------------------------------------------------------------
local keys
if g_params["KeyList"] then
    keys = csv_to_tbl(g_params["KeyList"])
else
	log_msg("parameter error",g_script)
	error()
end

----------------------------------------------------------------------
-- TargetDate パラメータが指定されていない場合にはスクリプトが起動
-- された日を集計対象開始日にして timestamp 変数に設定する。
-- Range パラメータが "day","2d","week","month" の場合に有効で、それ以外
-- の場合には timestamp は後で上書きされる
----------------------------------------------------------------------
local timestamp -- 一日以上の集計期間を指定する場合の開始日付時刻
local now = os.date "*t"
if g_params["TargetDate"] then
    timestamp = g_params["TargetDate"] .. " 0:0:0"
else
	timestamp = string.format("%4.4d/%2.2d/%2.2d 0:0:0",now["year"],now["month"],now["day"])
end
local interval,count,days
if not g_params["Range"] then g_params["Range"] = "day" end -- デフォルトは1日間の集計を行う

----------------------------------------------------------------------
-- Range, Interval パラメータから集計間隔と集計データ数を決定
----------------------------------------------------------------------

---------------
-- 1日間集計
---------------
if g_params["Range"] == "day" then	
	days = 1
	if not g_params["Interval"] then
		interval = 300
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor((days * 24 * 3600)/interval)
end
---------------
-- 2日間集計
---------------
if g_params["Range"] == "2d" then
	if not g_params["TargetDate"] then -- 集計対象日が未指定の場合には現在日から 1日前に設定する
		local stat,y,m,d,h,min,s = str_to_datetime(timestamp)
		if not stat then error() end
		local stat,y,m,d = inc_day(-1,y,m,d)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d 0:0:0",y,m,d)
	end
	days = 2
	if not g_params["Interval"] then
		interval = 600
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor((days * 24 * 3600)/interval)
end
---------------
-- 1週間集計
---------------
if g_params["Range"] == "week" then
	if not g_params["TargetDate"] then -- 集計対象日が未指定の場合には現在日から 6日前に設定する
		local stat,y,m,d,h,min,s = str_to_datetime(timestamp)
		if not stat then error() end
		local stat,y,m,d = inc_day(-6,y,m,d)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d 0:0:0",y,m,d)
	end
	days = 7
	if not g_params["Interval"] then
		interval = 3600
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor((days * 24 * 3600)/interval)
end
---------------
-- 1月間集計
---------------
if g_params["Range"] == "month" then
	local stat,y,m,d,h,min,s = str_to_datetime(timestamp)
	if not stat then error() end
	timestamp = string.format("%4.4d/%2.2d/%2.2d 0:0:0",y,m,1) -- 検索対象日を月初め1日に変更
	stat,days = days_in_month(y,m)	-- 集計対象日数を対象月に含まれる日数に設定
	if not stat then error() end
	if not g_params["Interval"] then
		interval = 14400
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor((days * 24 * 3600)/interval)
end
---------------
-- 1時間集計
---------------
if g_params["Range"] == "hour" then
	if (not g_params["TargetDate"]) and (not g_params["TargetTime"]) then -- 集計対象日と時間の両方が未指定の場合は１時間前の時刻に設定
		local stat,y,m,d,h,min,s = inc_second(-3600,now["year"],now["month"],now["day"],now["hour"],now["min"],0)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",y,m,d,h,min,s)
	end
	if (not g_params["TargetDate"]) and g_params["TargetTime"] then -- 集計対象時間のみ指定の場合は現在日に設定
		timestamp = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTime"]
	end
	if g_params["TargetDate"] and (not g_params["TargetTime"]) then -- 集計対象日のみ指定の場合は 0:0:0 に設定
	    timestamp = g_params["TargetDate"] .. " 0:0:0"
	end
	if g_params["TargetDate"] and g_params["TargetTime"] then -- 集計対象日と時間の両方を指定の場合
	    timestamp = g_params["TargetDate"] .. " " .. g_params["TargetTime"]
	end
	if not g_params["Interval"] then
		interval = 30
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor(3600/interval)
end
---------------
-- 6時間集計
---------------
if g_params["Range"] == "6h" then
	if (not g_params["TargetDate"]) and (not g_params["TargetTime"]) then -- 集計対象日と時間の両方が未指定の場合は6時間前の時刻に設定
		local stat,y,m,d,h,min,s = inc_second(-21600,now["year"],now["month"],now["day"],now["hour"],now["min"],0)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",y,m,d,h,min,s)
	end
	if (not g_params["TargetDate"]) and g_params["TargetTime"] then -- 集計対象時間のみ指定の場合は現在日に設定
		timestamp = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTime"]
	end
	if g_params["TargetDate"] and (not g_params["TargetTime"]) then -- 集計対象日のみ指定の場合は 0:0:0 に設定
	    timestamp = g_params["TargetDate"] .. " 0:0:0"
	end
	if g_params["TargetDate"] and g_params["TargetTime"] then -- 集計対象日と時間の両方を指定の場合
	    timestamp = g_params["TargetDate"] .. " " .. g_params["TargetTime"]
	end
	if not g_params["Interval"] then
		interval = 60
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor(21600/interval)
end
---------------
-- 5分間集計
---------------
if g_params["Range"] == "5m" then
	if (not g_params["TargetDate"]) and (not g_params["TargetTime"]) then -- 集計対象日と時間の両方が未指定の場合は5分前の時刻に設定
		local stat,y,m,d,h,min,s = inc_second(-300,now["year"],now["month"],now["day"],now["hour"],now["min"],0)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",y,m,d,h,min,s)
	end
	if (not g_params["TargetDate"]) and g_params["TargetTime"] then -- 集計対象時間のみ指定の場合は現在日に設定
		timestamp = string.format("%4.4d/%2.2d/%2.2d ",now["year"],now["month"],now["day"]) .. g_params["TargetTime"]
	end
	if g_params["TargetDate"] and (not g_params["TargetTime"]) then -- 集計対象日のみ指定の場合は 0:0:0 に設定
	    timestamp = g_params["TargetDate"] .. " 0:0:0"
	end
	if g_params["TargetDate"] and g_params["TargetTime"] then -- 集計対象日と時間の両方を指定の場合
	    timestamp = g_params["TargetDate"] .. " " .. g_params["TargetTime"]
	end
	if not g_params["Interval"] then
		interval = 2
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor(300/interval)
end
---------------
-- 2週間集計
---------------
if g_params["Range"] == "2w" then
	if not g_params["TargetDate"] then -- 集計対象日が未指定の場合には現在日から 13日前に設定する
		local stat,y,m,d,h,min,s = str_to_datetime(timestamp)
		if not stat then error() end
		local stat,y,m,d = inc_day(-13,y,m,d)
		if not stat then error() end
		timestamp = string.format("%4.4d/%2.2d/%2.2d 0:0:0",y,m,d)
	end
	days = 14
	if not g_params["Interval"] then
		interval = 7200
	else
		interval = tonumber(g_params["Interval"])
	end
	count = math.floor((days * 24 * 7200)/interval)
end

if count > 10000 then -- 集計計算に時間がかかりすぎるため、エラーにする
	log_msg("too many records",g_script)
	error()
end

----------------------------------------------------------------------
-- 集計に使用する計算方法のデフォルト設定
----------------------------------------------------------------------
if not g_params["Summary"] then g_params["Summary"] = "mean" end

----------------------------------------------------------------------
-- データ集計
----------------------------------------------------------------------
log_msg(string.format("start_datetime: %s interval: %d count: %d summary: %s",
							timestamp,interval,count,g_params["Summary"]),g_script)
local sum
local series_json = '['
local label_json = '['
local first_series = true
for k,v in ipairs(keys) do
	log_msg("calculating: " .. v,g_script)
	if first_series then
		first_series = false
	else
		series_json = series_json .. ","
		label_json = label_json .. ","
	end

	if g_params["Summary"] == "cumulative" then
		sum = 0	-- 累計値用に初期化
	end

	---------------------------
	-- キー(シリーズ)毎の集計
	---------------------------
	local stat,datetime,sample,total,mean,max,min,first = fastdb_summary(v,timestamp,interval,count)
	if not stat then error() end
	series_json = series_json .. '['
	label_json = label_json .. '{"label":"' .. v .. '"}' 
	local first_item = true
	for key,val in ipairs(datetime) do
		if g_params["Summary"] == "mean" then
			sum = mean[key]		-- 集計単位期間内の平均データを使用
		end
		if g_params["Summary"] == "total" then
			sum = total[key]	-- 集計単位期間内の合計データを使用
		end
		if g_params["Summary"] == "cumulative" then
			sum = sum + total[key]	-- 集計単位期間内の合計データを累計する
		end
		if g_params["Summary"] == "first" then
			sum = first[key]	-- 集計単位期間内の最初のデータを使用
		end

		if first_item then
			first_item = false
		else
			series_json = series_json .. ","
		end

		if (g_params["Summary"] == "mean") or (g_params["Summary"] == "total") or (g_params["Summary"] == "first") then
			if sample[key] > 0 then		-- 集計単位期間内にデータが存在しない場合にはnullレコードを出力
				series_json = series_json .. string.format('["%s",%g]',val,sum)
			else
				series_json = series_json .. string.format('["%s",null]',val)
			end
		end
		if (g_params["Summary"] == "cumulative") then -- 集計単位期間内にデータが存在しない場合でも累計値は常に出力
			series_json = series_json .. string.format('["%s",%g]',val,sum)
		end

	end
	series_json = series_json .. ']'
end
series_json = series_json .. ']'
label_json = label_json .. ']'

--------------------------------------------
-- リターンパラメータに JSON 文字列を設定
--------------------------------------------
script_result(g_taskid,"SeriesList",series_json)
script_result(g_taskid,"LabelList",label_json)

