
--[[

●機能概要

ST7032iコントローラを使用した LCD ディスプレイ(8x2)に文字列を表示する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイスノード名							"ESP32Node1"

bus         I2C バス(ポート)番号                                    "1"
            "0" または "1"を指定、省略時は "0" を使用する

line		msg パラメータに指定した文字列を表示する LCDデバイスのライン番号を
			指定する "0" または "1"
			このパラメータが省略されていて、かつ data に指定する文字列長が
			8 文字以上の場合には自動的に２行目に折り返して表示する
																	"1"

data		LCD に表示する文字列									"Hello World!!"
			16文字(バイト)以内

clear		このパラメータを指定すると最初にLCD 表示内容をクリアする
			値は何でも構わない										"1"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

●備考


●変更履歴

2018/08/27	MGCP デバイス I2C操作用に移植

2014/04/23	初版作成

abs_agent	copyright(c) All Blue System


]]

local p_bus = 0			-- I2Cポート
local p_slave_addr = "3E"	-- LDCデバイス I2Cアドレス
local p_node

-------------------------
-- パラメータチェック
-------------------------

if g_params["node"] then
	p_node = g_params["node"]
else
	error()
end

if g_params["bus"] then
	p_bus = tonumber(g_params["bus"])
end

local p_line
if (g_params["line"]) then
	p_line = tonumber(g_params["line"])
end


mgcp_ST7032i_LCD8X2_try_init(p_node,p_bus,p_slave_addr)

if (g_params["clear"]) then
	mgcp_ST7032i_LCD8X2_clear(p_node,p_bus,p_slave_addr)
end

if (g_params["data"]) then
	mgcp_ST7032i_LCD8X2_display(p_node,p_bus,p_slave_addr,g_params["data"],p_line)
end
