
--[[

●機能概要

I2C バスに接続した温度センサー(TMP102) の値を取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイスノード名							"ESP32Node1"

bus         I2C バス(ポート)番号                                    "1"
            "0" または "1"を指定、省略時は "0" を使用する

●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
temperature     センサーから取得した摂氏温度                     "12.5"
                                                                 "-25.0"
●備考

●変更履歴

2014/04/23	初版作成

abs_agent	copyright(c) All Blue System

]]

local bus = 0			-- I2Cポート
local slave_addr = "48"
local node

-------------------------
-- パラメータチェック
-------------------------
if g_params["node"] then
	node = g_params["node"]
else
	error()
end

if g_params["bus"] then
    bus = tonumber(g_params["bus"])
end

-----------------------------------------------------------------------
-- TMP102温度レジスタの値を取得する
-- pointer register 0x00 をセットした後、2 バイトのレジスタ値を取得する
-----------------------------------------------------------------------
local stat,result = mgcp_i2c_write_read(node,bus,slave_addr,"00",2)
if not stat then error() end

---------------------------------------
-- 温度レジスタ値から摂氏温度を計算する
---------------------------------------
local reg = {}
reg = hex_to_tbl(result)

local temp_int = bit_lshift(reg[1],4) + bit_rshift(reg[2],4)
local temperature = 0.0625 * bit_tosigned(temp_int,12)
script_result(g_taskid,"temperature",string.format("%3.1f",temperature))

