
--[[

●機能概要

M5Stack BASICデバイス内蔵スピーカでブザー音を出力する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイス(M5Stack)ノード名					"M5_N001"

state		ブザー(PWM#0 PWM出力)状態を指定する。パラメータ省略時は "off" になる
			"off"
			"on"
			"blink"

●リターンパラメータ

無し

●備考

M5Stack デバイスでは下記の初期設定値で PWM#0 出力が有効になっています。
PWM#0 は GPIO#25 経由でM5Stack 内蔵スピーカに接続されています。

"ch":0,"pwm_use":1,"servo_use":0,"gpio":25,"resolution":13,"freq":1760,"duty":0

●変更履歴

2021/6/26	blink フラグと duty 値変更を1コマンドで実行するようにした mgcp ver1.3b05 以降

2021/4/19	初版作成

abs_agent	copyright(c) 2021 All Blue System

]]

local p_node
local p_state
local ch = 0
local vol = 30	-- ブザー音量、Duty値による音量調整で 0-4000 程度まで指定可能だが音量は目安。

if g_params["node"] then
	p_node = g_params["node"]
else
	error()
end

if g_params["state"] then
	p_state = g_params["state"]
else
	p_state = "off"
end

if p_state == "off" then
	local cmd = {}
	cmd["command"] = "pwm_blink"
	cmd["ch"] = ch
	cmd["flag"] = "off"
	cmd["duty"] = 0
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end

if p_state == "on" then
	local cmd = {}
	cmd["command"] = "pwm_blink"
	cmd["ch"] = ch
	cmd["flag"] = "off"
	cmd["duty"] = vol
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end

if p_state == "blink" then
	local cmd = {}
	cmd["command"] = "pwm_blink"
	cmd["ch"] = ch
	cmd["flag"] = "on"
	cmd["duty"] = vol
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end
