
--[[

*********************************************************************************
MGCP イベントデータを受信時のイベントハンドラスクリプト。
MQTT_PUBLISH イベントハンドラから別スレッドでこのスクリプトが呼び出されます。
*********************************************************************************

EVENT スクリプト起動時に渡されるパラメータ
---------------------------------------------------------------------------------
キー値          値                                                      値の例
---------------------------------------------------------------------------------

endpoint		MGCP データを受信した MQTT エンドポイント
				ClientID または Title名が入る							"endpoint1"

sender			イベント送信元の MGCP デバイスノード名					"STICK001"

recipient		イベント送信時に指定された送信対象ノード名
				"_ALL_" または MGCP デバイスノード名が入る				"_ALL_"

device			MGCP デバイスタイプを表す文字列。下記のデバイスタイプ以外から
				送信されてきた場合にはこのパラメータに値は入らない。

				"STD"
				"M5Stack"
				"M5Stick"
				"AtomMatrix"
				"AtomLite"												"M5Stick"

type			イベント種別を表す文字列

				"startup"
				"heartbeat"
				"gpio_change_detect"
				"gpio_sampling"								"gpio_change_detect"

data			イベントデータ。JSON 形式の文字列が格納されている。

						{"seq":1,"gpio_val":"A004634A8B","gpio_diff":"2000000000"}
				

]]

--log_msg("start..")
local event_json_tbl = g_json.decode(g_params["data"])

-------------------
-- startup
-------------------
if g_params["type"] == "startup" then

	-- ノードのリブートカウンタを保持する $REBOOT_<Node> グローバル共有変数の更新。
	-- ノードに接続されたセンサなどのデバイス初期化が必要かどうかを判断するときに利用する。
	if event_json_tbl.reboot then
		set_shared_data('$REBOOT_' .. g_params["sender"],event_json_tbl.reboot,true) 
	end
end

-------------------
-- heartbeat
-------------------
if g_params["type"] == "heartbeat" then 

	-- 全ノード名リストを保持している $MGCP_NodeList の更新
	add_shared_strlist('$MGCP_NodeList',g_params["sender"],true)

	-- 全エンドポイント名リストを保持している $MGCP_EndPointList の更新
	add_shared_strlist('$MGCP_EndPointList',g_params["endpoint"],true)

	-- 各エンドポイントに接続されているノード名リスト $MGCP_Node_<EndPoint> の更新
	add_shared_strlist('$MGCP_Node_' .. g_params["endpoint"],g_params["sender"],true)
	if event_json_tbl.mac then -- MAC アドレス用のノード名も同様に  $MGCP_Node_<EndPoint>に登録
		add_shared_strlist('$MGCP_Node_' .. g_params["endpoint"],event_json_tbl.mac,true)
	end

	-- ノードのデバイスタイプを設定。グローバル共有変数を使用するため、ノード名が重複する場合には正確ではない
	set_shared_data("$TYPE_" .. g_params["sender"],g_params["device"])

	-- ノード名毎の MACアドレスリストを保持している $MAC_<Node> リストの更新。
	-- 通常このリストには１つのエントリしか入らないが、MQTTブローカーに２つ以上の同一ノード名のデバイスが接続
	-- していた場合には２つ以上のエントリが入ることになる。
	-- agent_mgcp コマンドのノードリスト表示時に、MAC アドレス表示が不要な場合にはコメントアウトしてください。
	-- コメントアウトすることで、ノードが多数存在するときのグローバル共有文字列リストによるリソース消費を軽減できる。
	if event_json_tbl.mac then
		add_shared_strlist('$MAC_' .. g_params["sender"],event_json_tbl.mac,true)
	end

	-- ノードの現在の IP アドレスを保持する $IP_<Node> グローバル共有変数の更新。
	-- ノードが再起動時にIP アドレスを変更した場合でも、IP 更新後に受信したイベントの IP アドレスを常に設定している
	-- agent_mgcp コマンドのノードリスト表示時に、IP アドレス表示が不要な場合にはコメントアウトしてください。
	-- コメントアウトすることで、ノードが多数存在するときのグローバル共有データによるリソース消費を軽減できる。
	if event_json_tbl.ip then
		set_shared_data('$IP_' .. g_params["sender"],event_json_tbl.ip,true) 
	end

	-- ノードのリブートカウンタを保持する $REBOOT_<Node> グローバル共有変数の更新。
	-- ノードに接続されたセンサなどのデバイス初期化が必要かどうかを判断するときに利用する。
	if event_json_tbl.reboot then
		set_shared_data('$REBOOT_' .. g_params["sender"],event_json_tbl.reboot,true) 
	end

	if event_json_tbl.dx_port then
		set_shared_data('$DX_PORT_' .. g_params["sender"],event_json_tbl.dx_port,true) 
	end

end

----------------------------------------------
-- ユーザー環境用スクリプトはここから
----------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "M5StackとMQTTを使用したメッセージ表示器の作成" をご覧ください
----------------------------------------------------------------------------------------------------------------------------
--[[
if g_params["type"] == "gpio_change_detect" then 
	local diff_bit_arr = hex_to_bit(event_json_tbl.gpio_diff)
	local val_bit_arr = hex_to_bit(event_json_tbl.gpio_val)
	-------------------------------------------------------------------------------------------------------
	-- M5Stack(左ボタン)/M5Stick-C(正面ボタン) 押した?
	-------------------------------------------------------------------------------------------------------
	local btn_gpio = -1
	if g_params["device"] == "M5Stick" then btn_gpio = 37 end
	if g_params["device"] == "M5Stack" then btn_gpio = 39 end
	if (btn_gpio ~= -1) and (diff_bit_arr[btn_gpio] == 1) and (val_bit_arr[btn_gpio] == 0) then
		local params = {}
		params["node"] = g_params["sender"]
		params["prev"] = "1" -- 前の過去メッセージ表示もしくは点滅中の LED 消灯
		script_exec("MGCP/DEVICE/M5_LCD_MSG",params)
	end
	-------------------------------------------------------------------------------------------------------
	-- M5Stack(中ボタン)/M5Stick-C(側面ボタン) 押した?
	-------------------------------------------------------------------------------------------------------
	local btn_gpio = -1
	if g_params["device"] == "M5Stick" then btn_gpio = 39 end
	if g_params["device"] == "M5Stack" then btn_gpio = 38 end
	if (btn_gpio ~= -1) and (diff_bit_arr[btn_gpio] == 1) and (val_bit_arr[btn_gpio] == 0) then
		local params = {}
		params["node"] = g_params["sender"]
		params["clear"] = "1" -- LCD クリアと過去メッセージ削除
		script_exec("MGCP/DEVICE/M5_LCD_MSG",params)
	end
	-------------------------------------------------------------------------------------------------------
	-- M5Stack(右ボタン)押した?
	-------------------------------------------------------------------------------------------------------
	local btn_gpio = -1
	if g_params["device"] == "M5Stack" then btn_gpio = 37 end
	if (btn_gpio ~= -1) and (diff_bit_arr[btn_gpio] == 1) and (val_bit_arr[btn_gpio] == 0) then
		local params = {}
		params["node"] = g_params["sender"]
		params["next"] = "1" -- 次の過去メッセージ表示もしくは点滅中の LED 消灯
		script_exec("MGCP/DEVICE/M5_LCD_MSG",params)
	end
end
]]
