----------------------------------------------------------------------------
-- LCD(AQM1248A) に BME280 で取得した温度ゲージをアニメーション表示
----------------------------------------------------------------------------
local center_x = 60
local center_y = 47
local outer_s = 40
local inner_s = 30
local label_font = "SHINONOME12_FONTX"
local font_size = 2
local lcd_ch = AQM1248A_DISP_BUFF_CH

function draw_tick(deg)
	graphic2_draw_ellipse_arc(lcd_ch,center_x,center_y,outer_s+5,outer_s,outer_s+5,outer_s,deg-1,deg+1,1,false,1)
end
function draw_needle(deg)
	graphic2_draw_ellipse_arc(lcd_ch,center_x,center_y,outer_s,inner_s,outer_s,inner_s,180,deg,1,true,0)
	graphic2_draw_ellipse_arc(lcd_ch,center_x,center_y,outer_s,inner_s-7,outer_s,inner_s-7,deg-2,deg+2,1,true,0)
end

fontx_try_init()
AQM1248A_try_init()
graphic2_clear(lcd_ch)

-- ゲージをバックグランドページに描画
graphic2_draw_ellipse_arc(lcd_ch,center_x,center_y,outer_s,inner_s,outer_s,inner_s,180,270,1,false,1)

-- 目盛をバックグランドページに描画
draw_tick(180)
draw_tick(180+90/4)
draw_tick(180+2*90/4)
draw_tick(180+3*90/4)
draw_tick(270)

-- ラベルをバックグランドページに描画
graphic2_print(lcd_ch,5,36,"0",1,label_font,1,0,1)
graphic2_print(lcd_ch,5,23,"10",1,label_font,1,0,1)
graphic2_print(lcd_ch,13,8,"20",1,label_font,1,0,1)
graphic2_print(lcd_ch,28,0,"30",1,label_font,1,0,1)
graphic2_print(lcd_ch,65,0,"40",1,label_font,1,0,1)

-- フォアグランドページをLCDに表示してバックグランドの内容で置き換える
AQM1248A_display(0,1)

-- 現在の温度取得
local BME280 = stat_check(script_exec2("RASPI/DEVICE/BME280_READ","",""))
local temperature = tonumber(BME280["temperature"])
temperature = temperature - 7.1 -- ボードの温度を考慮して実際の気温に修正(だいたい)
if temperature > 40 then temperature = 40 end
if temperature < 0 then temperature = 0 end

-- バックグランドの描画内容に重ねてゲージとニードルをアニメーション表示させる
local step = 20
for i = 1,step do
	local tv = temperature*i/step
	draw_needle(180+90*tv/40)
	graphic2_print(lcd_ch,50,24,string.format('%.1f℃',temperature),1,label_font,font_size,0,0)
	AQM1248A_display(0,1)
	wait_time(10)
end
