
--[[

●機能概要

LCD(ST7789)ディスプレイにアナログ時計を表示する

●リクエストパラメータ

無し

●備考

このスクリプトは終了しないので別スレッドで起動して下さい。

●変更履歴

2022/11/28    初版作成

abs_agent	copyright(c) All Blue System

]]

-- 画像ファイル(RGB)をディスプレイバッファにロードした後、画像データ部分のカラー値配列を返す関数
function load_image(name,w,h,page)
	stat_check(graphic2_clear(ST7789_DISP_BUFF_CH,page))
	stat_check(graphic2_load_rgb_file(ST7789_DISP_BUFF_CH,g_dir .. '/' .. name,w,h,0,0,page))
	return stat_check(graphic2_get_image(ST7789_DISP_BUFF_CH,0,0,w,h,page))
end

local bck_w = 240			-- 文字盤画像の幅
local bck_h = 240			-- 文字盤画像の高さ
local center_x = 122.2		-- 文字盤上の針軸 x座標
local center_y = 119.6		-- 文字盤上の針軸 y座標

local short_w = 24			-- 短針画像の幅
local short_h = 71			-- 短針画像の高さ
local short_src_x = 12.06	-- 短針の回転位置 x座標 (短針画像左端からの相対座標)
local short_src_y = 59.31	-- 短針の回転位置 y座標 (短針画像上端からの相対座標)

local long_w = 19			-- 長針画像の幅
local long_h = 99			-- 長針画像の高さ
local long_src_x = 9.63	-- 長針の回転位置 x座標 (長針画像左端からの相対座標)
local long_src_y = 90.69	-- 長針の回転位置 y座標 (長針画像上端からの相対座標)

local sec_w = 19			-- 秒針画像の幅
local sec_h = 116			-- 秒針画像の高さ
local sec_src_x = 9.25		-- 秒針の回転位置 x座標 (秒針画像左端からの相対座標)
local sec_src_y = 89.13	-- 秒針の回転位置 y座標 (秒針画像上端からの相対座標)

local a_color = rgb565(0xff,0xff,0xff) -- 針画像中の透過色にするカラー(WHITE)

ST7789_try_init()

local short_img = load_image('短針_24x71.raw',short_w,short_h,1)	-- 短針画像ファイルをロードして画像のカラー値配列を得る
local long_img = load_image('長針_19x99.raw',long_w,long_h,1)		-- 長針画像ファイルをロードして画像のカラー値配列を得る
local sec_img = load_image('秒針_19x116.raw',sec_w,sec_h,1)		-- 秒針画像ファイルをロードして画像のカラー値配列を得る
load_image('背景_240x240.raw',bck_w,bck_h,1)						-- 文字盤画像ファイルをディスプレイバッファ中の page#1 にロードしたままにする。
local prev_sec_deg	= -1											-- 画面の描画頻度を減らすために、秒針の位置が変化しているかを調べる
while true do
	local year,mon,day,hour,minute,sec,millisec = stat_check(now_datetime())
	local hour_deg = 360 * (((hour % 12) + minute / 60) / 12)	-- 短針の角度を(時+分)の解像度で計算する。
	local min_deg = 360 * ((minute + sec / 60) / 60)			-- 長針の角度を(分+秒)の解像度で計算する。
	local sec_deg = 360 * (sec / 60)								-- 秒針の角度を計算
	if prev_sec_deg ~= sec_deg then -- 針の位置は変化した?
		prev_sec_deg = sec_deg
		-- 文字盤が描画済みのディスプレイバッファpage#0 上に全ての針画像を描画する。
		stat_check(graphic2_draw_image(ST7789_DISP_BUFF_CH,short_img,short_w,short_h,center_x,center_y,short_src_x,short_src_y,1,1,hour_deg,a_color,0))
		stat_check(graphic2_draw_image(ST7789_DISP_BUFF_CH,long_img,long_w,long_h,center_x,center_y,long_src_x,long_src_y,1,1,min_deg,a_color,0))
		stat_check(graphic2_draw_image(ST7789_DISP_BUFF_CH,sec_img,sec_w,sec_h,center_x,center_y,sec_src_x,sec_src_y,1,1,sec_deg,a_color,0))
		ST7789_display(0,1) -- ディスプレイバッファ page#0 の内容を LCD に転送後、文字盤画像のみが描画されている page#1 の内容を page#0 にコピーする。
	end
	wait_time(100) -- 更新チェック間隔
end
