
--[[

●機能概要

I2C バスに接続した温度センサー(TMP102) の値を取得する

●リクエストパラメータ
---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
bus             I2C バス番号                                     "1"
                "0" または "1"を指定、省略時は "1" を使用する

●リターンパラメータ
---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
temperature     センサーから取得した摂氏温度                     "12.5"
                                                                 "-25.0"

●備考


●使用条件・免責事項

このスクリプトファイルは自由にお客様が複製、改変を行うことができます。またお客様のアプリケーションと
共にこのファイルを配布することができます。

このスクリプトは公開されているデバイス仕様書を元に、オールブルーシステムがサンプル実装したものです。
お客様がこのファイルを利用される場合には、お客様または第三者に損害が生じた場合でも、
オールブルーシステムは損害賠償その他一切の責任を負担しません。

●変更履歴

2016/05/10  abs_agent RASPI H/W モジュール用に移植

2014/04/23    初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

local slave_addr = "48"
local bus = 1

-------------------------
-- パラメータチェック
-------------------------
if g_params["bus"] then
    bus = tonumber(g_params["bus"])
end

----------------------------------------
-- 12 bit 幅の2の補数を符号付整数に変換
----------------------------------------
function calc_2comp(val)
    if(bit_and(val,0x800) ~= 0) then
        return -1 * (bit_and(bit_not(val),0xfff) + 1)
    else
        return val
    end
end

-----------------------------------------------------------------------
-- TMP102温度レジスタの値を取得する
-- pointer register 0x00 をセットした後、2 バイトのレジスタ値を取得する
-----------------------------------------------------------------------
local stat,result = raspi_i2c_write(bus,slave_addr,"00",2)
if not stat then error() end

---------------------------------------
-- 温度レジスタ値から摂氏温度を計算する
---------------------------------------
local reg = {}
reg = hex_to_tbl(result)

local temp_int = bit_lshift(reg[1],4) + bit_rshift(reg[2],4)
local temperature = 0.0625 * calc_2comp(temp_int)
script_result(g_taskid,"temperature",string.format("%3.1f",temperature))
