
--[[

●機能概要

環境センサーボード(スイッチサイエンス社製)に搭載している LED をコントロールします。
グローバル共有変数にセットされたアラーム状態を示すフラグ値に対応して、
ボード上の LEDの点灯、消灯、点滅などを指定できます。


●参照するグローバル共有変数

---------------------------------------------------------------------------------
キー値            値                            　 操作するGPIO# と設定値                
---------------------------------------------------------------------------------
"alarm_red"     "off" or "" (変数未定義)       GPIO#22  High                
                "on"                           GPIO#22  Low
                "blink"                        GPIO#22  Low-High 繰り返し

"alarm_blue     "off" or "" (変数未定義)       GPIO#17  High                
                "on"                           GPIO#17  Low
                "blink"                        GPIO#17  Low-High 繰り返し

"alarm_green"   "off" or "" (変数未定義)       GPIO#27  High                
                "on"                           GPIO#27  Low
                "blink"                        GPIO#27  Low-High 繰り返し

●備考

このスクリプトは無限ループに入って終了しないので、必ず別スレッドで起動してください。

●変更履歴

2017/03/12    環境センサーボード用に移植

2016/08/23    初版作成

copyright(c) 2017 All Blue System

]]

-- ２重起動防止用チェック
if not exclusive_check(g_script) then
    log_msg("*ERROR* exclusive_check() failed. script = " .. g_script,file_id)
    return
end
log_msg("start..   TaskID = " .. g_taskid,g_script)

-- GPIO ピン番号設定
local RED_pin = 22
local BLUE_pin = 17
local GREEN_pin = 27
local SW_pin = 24
local long_timer_interval = 640      -- LED 点滅間隔、グローバル共有変数のチェック間隔(ms)

-- グローバル共有変数の最新の値のキャッシュ, beep2 間隔でグローバル共有変数から最新の値をロードして更新される
local alarm_red = ""
local alarm_blue = ""
local alarm_green = ""

local led_stat = false            -- LEDとブザー(beep2)の High - Low 繰り返しの現在のステート値

-- LED と スイッチを接続する GPIO のモードを初期設定する。
if not raspi_gpio_config(RED_pin,"output","off") then error() end 
if not raspi_gpio_config(BLUE_pin,"output","off") then error() end 
if not raspi_gpio_config(GREEN_pin,"output","off") then error() end 
if not raspi_gpio_config(SW_pin,"input","off") then error() end

-- スイッチ入力を検知するために RASPI_CHANGE_DETECT イベントを有効にする
if not raspi_change_detect(SW_pin,true) then error() end


-- long_timer_interval 間隔で実行
function long_cyc_task()
    local stat

    -- 最新のグローバル共有変数の値をキャッシュにロード 
    stat,alarm_red = get_shared_data("alarm_red")
    stat,alarm_blue = get_shared_data("alarm_blue")
    stat,alarm_green = get_shared_data("alarm_green")
 
    -- LED の状態を反転する
    led_stat = not led_stat

    -- LED の GPIO 出力
    if alarm_red == "on" then
        if not raspi_gpio_write(RED_pin,false) then error() end 
    elseif alarm_red == "blink" then
        if not raspi_gpio_write(RED_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(RED_pin,true) then error() end 
    end

    if alarm_blue == "on" then
        if not raspi_gpio_write(BLUE_pin,false) then error() end 
    elseif alarm_blue == "blink" then
        if not raspi_gpio_write(BLUE_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(BLUE_pin,true) then error() end 
    end

    if alarm_green == "on" then
        if not raspi_gpio_write(GREEN_pin,false) then error() end 
    elseif alarm_green == "blink" then
        if not raspi_gpio_write(GREEN_pin,led_stat) then error() end 
    else
        if not raspi_gpio_write(GREEN_pin,true) then error() end 
    end

end

-- グローバル変数の値を定期的に監視してLED に出力するための、監視ループに入る
while true do -- 無限ループを停止させる場合には script_kill() または "agent_task -k <taskid>" コマンドを使用する
    wait_time(long_timer_interval)
    long_cyc_task()
end

