
--[[

●機能概要

環境センサーボード(スイッチサイエンス社製)に搭載している BME280 センサと光センサの
測定値を MQTT ブローカに送信する。

同時にローカルコンピュータのインメモリデータベース FASTDBにもデータを保存する。

●備考

abs_agent 設定ファイルに MQTT ブローカ接続用のエンドポイントを下記例を参考に作成してください。
ブローカのホスト名や接続ユーザー名、パスワード、ClientID等は環境に合わせて変更します。
エンドポイントタイトル名を変更した場合には、このスクリプト中のタイトル名部分も変更して
ください。

abs_agent.xml ファイル中の MQTTサービスモジュール設定例：

    <MQTT>
      <AutoOnline type="boolean">True</AutoOnline>
      <KeepAliveTimer type="integer">60</KeepAliveTimer>
      <EndPointList>
        <Item>
          <Title>センサーデータ送受信</Title>
          <ClientID>abs9k:93501-raspi3</ClientID>
          <BrokerHostName>192.168.100.14</BrokerHostName>
          <PortNumber>1883</PortNumber>
          <AutoSubscribeTopicList>/EnvSensor/+</AutoSubscribeTopicList>
          <AutoSubscribeQoSList>0</AutoSubscribeQoSList>
          <UserName/>
          <Password/>
          <WillTopic/>
          <WillMessage/>
          <WillQoS>0</WillQoS>
          <WillRetain>False</WillRetain>
          <RecvBuffInit>2048</RecvBuffInit>
          <DetailLog>False</DetailLog>
        </Item>
      </EndPointList>
    </MQTT>


●変更履歴

2022/8/23   温度センサの補正計算を追加。
			stat_check() によるステータスチェックに変更。

2017/5/1    初版作成

copyright(c) 2022 All Blue System

]]

--local bme280_temp_adj = 7.1 -- BME280で測定した温度の補正値(環境センサボードの放熱分を減算する)
local bme280_temp_adj = 0

---------------------------------------------------------------------------------
-- BME280 センサー
---------------------------------------------------------------------------------
local BME280 = stat_check(script_exec2("RASPI/DEVICE/BME280_READ",{}))
temp_adj = tonumber(BME280["temperature"]) - bme280_temp_adj

---------------------------------------------------------------------
-- 光センサー
---------------------------------------------------------------------
local ENVSENSOR = stat_check(script_exec2("RASPI/DEVICE/ENVSENSOR_LIGHT_READ",{}))

---------------------------------------------------
-- インメモリデータベースに測定データを保存する
---------------------------------------------------
stat_check(fastdb_add("温度",temp_adj))
stat_check(fastdb_add("気圧",tonumber(BME280["pressure"])))
stat_check(fastdb_add("湿度",tonumber(BME280["humidity"])))
stat_check(fastdb_add("明るさ",tonumber(ENVSENSOR["light"])))

---------------------------------------------
-- MQTT ブローカに測定データを送信する
---------------------------------------------
local end_point = "センサーデータ送受信"
local topic = "/EnvSensor/" .. g_hostname
local qos = 0
local json_str = '{'
json_str = json_str ..        '"temperature":' .. tostring(temp_adj)
json_str = json_str .. "," .. '"pressure":' .. BME280["pressure"]
json_str = json_str .. "," .. '"humidity":' .. BME280["humidity"]
json_str = json_str .. "," .. '"light":' .. ENVSENSOR["light"]
json_str = json_str .. '}'

stat_check(mqtt_publish(end_point,topic,json_str,qos))

