
--[[

●機能概要

http://newsapi.org で提供されている最新のニュースヘッドラインを取得して、文字列リストに格納する。
このスクリプトを PERIODIC_TIMER イベントハンドラから定期的にコールすることで、最新の
ニュース記事タイトルを文字列に保存できる。

このスクリプトは RASPI/SCROLLPHATHD_BULLETIN_BOARD_TASK スクリプトと組み合わせることで、
ニュース記事を LED マトリックス表示器に順にスクロール表示させることができます。

●設定するグローバル共有文字列リスト変数

"SCROLLPHATHD_STRINGS"

●備考

********
**重要** 
********

使用前に必ず http://newsapi.org のライセンス条項を読んで適切な apiKey を取得してください。
取得した apiKey はスクリプト中のパラメータ設定部分に設定してください。

●変更履歴

2018/04/13    初版作成

copyright(c) All Blue System

]]

log_msg("start..",g_script)

----------------------------
-- タイムスタンプ文字列作成
----------------------------
local now = os.date "*t"
local stat,wday = day_of_week(now["year"],now["month"],now["day"])
if not stat then error() end
local weekstr
if wday == 1 then 
	weekstr = "Sun"
elseif wday == 2 then
	weekstr = "Mon"
elseif wday == 3 then
	weekstr = "Tue"
elseif wday == 4 then
	weekstr = "Wed"
elseif wday == 5 then
	weekstr = "Thu"
elseif wday == 6 then
	weekstr = "Fri"
elseif wday == 7 then
	weekstr = "Sat"
end
local timestamp_str = string.format("%2.2d:%2.2d %s",now["hour"],now["min"],weekstr)

-------------------------------------
-- WebAPI アクセス時のパラメータ設定
-------------------------------------
local host = "newsapi.org"
local port = 80
local path = "/v2/top-headlines"
local header = {}
local params = {}
params["country"] = "us"
params["category"] = "general"

-- *****************************************************************
-- ***** 必ず下記の apiKey 文字列を設定してから起動すること!!! *****
-- *****************************************************************
params["apiKey"] = "********"

-------------------------------------
-- 表示器用文字列リストをクリアする
-------------------------------------
local listvar = "SCROLLPHATHD_STRINGS"
if not clear_shared_strlist(listvar) then error() end

----------------
-- Web API 実行
----------------
local stat,rpl = http_get(host,port,path,params,header,true)
if not stat then
	if not add_shared_strlist(listvar,"*ERROR* HTTP-GET " .. host .. " failed.") then error() end
	return
end

-----------------------------------------------------------------------
-- Web APIで返された JSON 文字列を解析して、表示器用文字列リストを作成
-----------------------------------------------------------------------
local rpl_json = g_json.decode(rpl)
if rpl_json.status ~= "ok" then
	if not add_shared_strlist(listvar,"*ERROR* WebAPI " .. host .. " failed, status = ".. rpl_json.status) then error() end
	return
end
local articles_max = #rpl_json.articles
for i = 1,articles_max,1 do
	if not add_shared_strlist(listvar,"[" .. rpl_json.articles[i].source.name .. "] " .. rpl_json.articles[i].title .. 
								" [" .. timestamp_str .. "]") then error() end
end
