
file_id = "RASPI_CHANGE_DETECT"

--[[

******************************************************************************

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
処理に時間がかかると、イベント処理の終了を待つサーバー側でタイムアウトが発生します。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しないイベントで、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************


RASPI_CHANGE_DETECT スクリプト起動時に渡される追加パラメータ
---------------------------------------------------------------------------------
キー値          値                                                      値の例
---------------------------------------------------------------------------------

BitNumList      GPIO の監視対象ビット中で変化したGPIO ピン番号が入る
                複数同時に変化した場合にはカンマ区切りのリストになる。　18,22,23
                0 から 53 までの整数が入る

BitValList      BitNumList に格納されている GPIO ピンの変化後の現在の値。
                カンマ区切りのリストで表される場合にはBitNumList のピン番号
                並びと値の並びが対応しています。                        0,1,1
                0 または 1 の値が入る


このスクリプト中で作成される配列
---------------------------------------------------------------------------------
変数名            説明
---------------------------------------------------------------------------------

change_bit[]    数値配列。キーが GPIO ピン番号の整数で、配列の値は
                GPIO の現在値を示す。 0(low) または 1(high) の整数値
                たとえば上記の BitNumList, BitValList の "値の例" の場合には
                change_bit[] 配列には下記の値が格納されます。
                
                change_bit[18] = 0
                change_bit[22] = 1
                change_bit[23] = 1
        
                ** メモ **
                Lua では存在しないキーの配列エントリにアクセスすると、
                値は nil が返ります。上記の例では change_bit[0] = nil です。

]]

log_msg("start..",file_id)

--------------------------
-- change_bit[] 配列作成
--------------------------
local arr_pos = csv_to_tbl(g_params['BitNumList'])
local arr_val = csv_to_tbl(g_params['BitValList'])
local change_bit = {}
for i,v in ipairs(arr_pos) do
    change_bit[tonumber(v)] = tonumber(arr_val[i])
end

----------------------------------------------
-- change_bit[] 配列内容確認のためログ出力
----------------------------------------------
for key,val in pairs(change_bit) do
    log_msg(string.format("change_bit[%d] = %d",key,val),file_id)
end

--------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は abs_agent ユーザーマニュアルをご覧ください
--------------------------------------------------------------------------------
--[[
if change_bit[23] then -- SW を操作した？
	if change_bit[23] == 0 then -- SW を押した状態
		local stat,cur_val = raspi_gpio_read(18)
		if not stat then error() end
		if not raspi_gpio_write(18,not cur_val) then error() end
	end
end
]]

--------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は abs_agent ユーザーマニュアルをご覧ください
--------------------------------------------------------------------------------
--[[
if change_bit[23] then -- SW を操作した？
	if change_bit[23] == 0 then -- SW を押した状態
		os.execute('/sbin/shutdown -h now &')
	end
end
]]

--------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は abs_agent ユーザーマニュアルをご覧ください
--------------------------------------------------------------------------------
--[[
if change_bit[23] then -- SW を操作した？
	local msg
	local clientid = "試験用MQTT接続"
	local topic = "/" .. g_hostname .. "/switch"
	if change_bit[23] == 0 then 
		msg = "ON"
	else
		msg = "OFF"
	end
	if not mqtt_publish(clientid,topic,msg,0) then error() end -- MQTTブローカに SW の状態を送信
end
]]

-------------------------------------------------------------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "WebSocket を使用した双方向通信リアルタイムアプリの作成" をご覧ください
-------------------------------------------------------------------------------------------------------------------------------------
--[[
if change_bit[23] then -- LED操作用 タクトSW を操作した？
	if change_bit[23] == 0 then -- SW を押した状態
		local stat,cur_val = raspi_gpio_read(18)
		if not stat then error() end
		if not raspi_gpio_write(18,not cur_val) then error() end
	end
end

if change_bit[18] then -- LED(GPIO#18) が変化した？
	local msg 
	if change_bit[18] == 1 then
		msg = '{"led":"on"}'
	else
		msg = '{"led":"off"}'
	end
	log_msg(msg,g_script)
	websocket_emit_text('led_websock',msg)
end
]]

----------------------------------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "MQTT接続のアラーム装置作成" をご覧ください
----------------------------------------------------------------------------------------------------------
--[[
if change_bit[23] then -- SW を操作した？
	local msg
	local clientid = "アラーム装置MQTT接続"
	local topic = "/alarm/switch/" .. g_hostname
	if change_bit[23] == 0 then 
		msg = '{"sw1":"on"}'
	else
		msg = '{"sw1":"off"}'
	end
	if not mqtt_publish(clientid,topic,msg,0) then error() end -- MQTTブローカに SW の状態を送信
end
]]

----------------------------------------------------------------------------------------------------------
-- スイッチサイエンス社製 環境センサーボード用のセンサー値表示アプリケーション設定
-- アプリケーションの詳細は RASPI/ENVSENSOR_DISPLAY_TASK スクリプトを参照してください
-- アプリ中で指定したページ切り替えを行うスイッチ(SW1)の入力を検出して、
-- アクティブページを示すグローバル共有変数をインクリメントする
----------------------------------------------------------------------------------------------------------
--[[
if change_bit[24] then -- 環境センサーボードの SW1 を操作した？
	if change_bit[24] == 0 then 
		local stat,val = inc_shared_data("ENVSENSOR_ACTIVE_PAGE")
		if tonumber(val) > 4 then -- SWを押すたびにアクティブなページ番号を 1..4 の順に設定する
			set_shared_data("ENVSENSOR_ACTIVE_PAGE","1")
		end
	end
end
]]


