
--[[
******************************************************************************
このスクリプトは abs_agent 起動時にコールされます
このスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
******************************************************************************
]]
log_msg("start..")

---------------------------------------------------------------
-- MQTT モジュールが有効な場合に全てのエンドポイント接続の開始
---------------------------------------------------------------
local mstat,module_stat = service_module_status()
if not mstat then error() end
if module_stat["MQTT"] then 
    script_fork_exec("MQTT_CONNECT_ALL","","")
end

--[[

認証を省略してアクセスするためのセッションを作成する。
下記のコメントを削除すると、create_session() ライブラリ関数をコールするときの
第一パラメータに記述したセッショントークンが abs_agent 起動時に自動作成されます。
第二パラメータを true にして、セッションを使用しない場合でも自動削除対象とならない
ようにしています。Web API から abs_agent をアクセスするときに、ログイン認証を
省略したい場合に、ここで指定したセッショントークン文字列を "session" URLパラメータ
に指定できます。

*** 重要 ***
コメントアウトする時には、セッショントークン文字列を任意の文字列に変更してください

]]
-- if not create_session("1234",true) then error() end

-----------------------------------------------------------------
-- FASTDB の過去データ保存期間を 30日に設定する例(default 10日)
-- FASTDB_PURGE.lua スクリプト内で使用する
-----------------------------------------------------------------
-- set_shared_data("$FASTDB_PURGE_BEFORE","30")

-----------------------------------------------------------------------------
-- リモートアクセスタイムアウトのリセット時間を3分に設定する例(default 10分)
-- REMOTE_LOST_HOSTS_RESET.lua スクリプト内で使用する
-----------------------------------------------------------------------------
-- set_shared_data("$REMOTE_LOST_HOSTS_CLEAR_TIME","3")

----------------------------------------------------------------------------------
-- GPIO#18 LED出力, GPIO#23 スイッチ入力設定
-- 下記のサンプルスクリプトの説明は abs_agent ユーザーマニュアルをご覧ください
----------------------------------------------------------------------------------
--[[
if not raspi_gpio_config(18,"output","off") then error() end 
if not raspi_gpio_config(23,"input","pullup") then error() end
if not raspi_change_detect(23,true) then error() end
]]

-------------------------------------------------------------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "WebSocket を使用した双方向通信リアルタイムアプリの作成" をご覧ください
-------------------------------------------------------------------------------------------------------------------------------------
--[[
if not raspi_gpio_config(18,"output","off") then error() end 
if not raspi_gpio_config(23,"input","pullup") then error() end
if not raspi_change_detect(23,true) then error() end -- スイッチ入力変化時にイベント発生
if not raspi_change_detect(18,true) then error() end -- LED出力変化時にイベント発生
]]

----------------------------------------------------------------------------------------------------------
-- Raspberry Pi GPIO アラームタスク起動
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "MQTT接続のアラーム装置作成" をご覧ください
----------------------------------------------------------------------------------------------------------
--[[
script_fork_exec("RASPI/ALARM_TASK","","")
]]

-----------------------------------------------------
-- スイッチサイエンス社製 環境センサーボード用設定
-----------------------------------------------------
--[[
raspi_gpio_config(18,"input","off")		-- SW3
raspi_gpio_config(23,"input","off")		-- SW2
raspi_gpio_config(24,"input","off")		-- SW1

raspi_gpio_config(4,"output","off")		-- LED4 Ir
raspi_gpio_config(17,"output","off")	-- LED3 BLUE
raspi_gpio_config(22,"output","off")	-- LED1 RED
raspi_gpio_config(25,"output","off")	-- LCD RS (AQM1284A)
raspi_gpio_config(27,"output","off")	-- LED2 GREEN

-- GPIO 初期値出力、全LED消灯, LCD RS->low
raspi_gpio_write(4,false)
raspi_gpio_write(17,true)
raspi_gpio_write(22,true)
raspi_gpio_write(25,false)
raspi_gpio_write(27,true)

-- アラームタスク起動
script_fork_exec("RASPI/ENVSENSOR_ALARM_TASK","","")

-- センサー値表示タスク起動
script_fork_exec("RASPI/ENVSENSOR_DISPLAY_TASK","","")

]]

-------------------------------------------------------------------------------------------------
-- PIMORONI製 ScrollpHAT HD (17x7 pixel)スクロール表示タスク起動設定(1つの文字列をスクロール表示)
-- RASPI/SCROLLPHATHD_BULLETIN_BOARD_TASK スクリプトとは同時に使用できませんので注意!!
-------------------------------------------------------------------------------------------------
--[[

-- グローバル共有変数 "SCROLLPHATHD_MESSAGE" に設定された文字列をスクロール表示するタスクを起動
script_fork_exec("RASPI/SCROLLPHATHD_DISPLAY_TASK","","")

-- abs_agennt 起動時に IP アドレスを表示する
local stat,info = script_exec2("OS/GET_IP","","")
if not set_shared_data("SCROLLPHATHD_MESSAGE","IP=" .. info["IPAddress"]) then error() end

]]

-------------------------------------------------------------------------------------------
-- PIMORONI製 ScrollpHAT HD (17x7 pixel)スクロール表示タスク起動設定(複数文字列を順に表示)
-- RASPI/SCROLLPHATHD_DISPLAY_TASK スクリプトとは同時に使用できませんので注意!!
-------------------------------------------------------------------------------------------
--[[

-- グローバル共有文字列リスト "SCROLLPHATHD_STRINGS" に設定された文字リストを順にスクロール表示するタスクを起動
script_fork_exec("RASPI/SCROLLPHATHD_BULLETIN_BOARD_TASK","","")

-- abs_agennt 起動時にホスト情報を表示する
local stat,mod_stat,config = service_module_status()
local stat,info = script_exec2("OS/GET_IP","","")
if not add_shared_strlist("SCROLLPHATHD_STRINGS","<<abs_agent ver" .. config["Version"] .. ">>") then error() end
if not add_shared_strlist("SCROLLPHATHD_STRINGS","hostname: " .. g_hostname) then error() end
if not add_shared_strlist("SCROLLPHATHD_STRINGS","ip_address: " .. info["IPAddress"]) then error() end

]]

