
--[[
****************************************************************************************************************

TCPサーバーにクライアントが接続するとワーカースレッドが作成され、その後ソケットからデータを受信すると
このイベントハンドラが実行される。受信するデータフォーマット(バイト数)は
abs_agent コンフィギュレーション中の NETSERVER 部分に記述したフォーマットに従う。

TCPSERVER_DATA イベントハンドラ中でリターンパラメータ EventResult を設定すると、ワーカースレッドから
接続中のクライアントにリプライパケットを送信する。EventResultを設定しなかった場合には送信しない。

ワーカースレッドが接続中のクライアントから引き続きパケットを送信した場合には、このイベントハンドラが
再び起動される。クライアント側からコネクションを切断した場合には、ワーカースレッドも終了する。

TCPSERVER_DATA イベントハンドラ中で error() 関数をコールしたり、シンタックスエラー等でスクリプトが強制終了
すると、ワーカスレッド側からコネクションを切断した後ワーカースレッドも終了する。

イベントハンドラ起動時に渡されるパラメータ:
---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
TCPSocketID     接続毎に付けられたユニークな文字列

IPAddress       クライアントの IP アドレス

EventData       受信したバイナリデータを16進数文字列に変換したもの


イベントハンドラ中で設定することで TCPクライアント側に送信されるリプライデータ:
---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
EventResult     クライアントに送信するバイナリデータを16進数文字列に変換したもの
				(オプション)

****************************************************************************************************************
]]
log_msg('received from ' .. g_params['IPAddress'] .. ' data = ' .. g_params['EventData'])


--[[

**************************************************
ModbusTCP プロトコルサンプル実装
**************************************************

●リクエストパケットフォーマット

req_arr[1] ：トランザクション識別子
req_arr[2] ：トランザクション識別子
req_arr[3] ：プロトコル識別子
req_arr[4] ：プロトコル識別子
req_arr[5] ：フィールド長（上位バイト）
req_arr[6] ：フィールド長（下位バイト）
req_arr[7] ：UnitID・スレーブアドレス
req_arr[8] ：ファンクションコード
req_arr[9] ：Data#0
..
..
req_arr[x] : Data#n


●リプライパケットフォーマット

rpl_arr[1] ：トランザクション識別子
rpl_arr[2] ：トランザクション識別子
rpl_arr[3] ：プロトコル識別子
rpl_arr[4] ：プロトコル識別子
rpl_arr[5] ：フィールド長（上位バイト）
rpl_arr[6] ：フィールド長（下位バイト）
rpl_arr[7] ：UnitID・スレーブアドレス(リクエストパケットのコピー)
rpl_arr[8] ：ファンクションコー(リクエストパケットのコピーとMSB にエラーステータスを含む)
rpl_arr[9] ：Data#0
..
..
rpl_arr[x] : Data#n

]]


local req_arr = hex_to_tbl(g_params["EventData"])
local rpl_arr = {}
if #req_arr < 6 then error() end

--error()

-- transaction ID
table.insert(rpl_arr,req_arr[1])
table.insert(rpl_arr,req_arr[2])
-- protocol ID
table.insert(rpl_arr,0x00)
table.insert(rpl_arr,0x00)
-- reply length
table.insert(rpl_arr,0x00)
table.insert(rpl_arr,0x05)
-- reply data
table.insert(rpl_arr,req_arr[7]) -- set UnitID
table.insert(rpl_arr,req_arr[8]) -- set function-code without errors
table.insert(rpl_arr,0x02)
table.insert(rpl_arr,0x00)
table.insert(rpl_arr,0x05)

script_result("EventResult",tbl_to_hex(rpl_arr))

