
--[[

******************************************************************************
* イベントハンドラスクリプト実行時間について                                 *
******************************************************************************

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
処理に時間がかかると、イベント処理の終了を待つサーバー側でタイムアウトが発生します。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しないイベントで、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************

WEBSOCKET_DATA スクリプト起動時に渡される追加パラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
Channel			クライアントが WebSocket 接続時に指定した URL パス名部分の
				Channel 文字列が格納される。パス名 := "/<channel>/<SessionToken>"
																		"my_app1"

SessionToken	クライアントが WebSocket 接続時に指定した URL パス名部分の
				SessionToken文字列が格納される。セッショントークン文字列は
				WebSocket 接続時に認証に使用した後は、WebSocket サーバー側
				では一切使用しない。このため、イベント発生時にこの
				セッショントークンが abs_agent 側に存在するかは関知して
				いない点に注意。パス名 := "/<channel>/<SessionToken>"
																		"ST04B20D83368F1C"

WebSocketID		WebSocket クライアント接続毎に付けられたユニークな文字列。
				このパラメータ値を websocket_emit_text(), websocket_emit_binary()
				ライブラリ関数の第一パラメータに指定すると、イベント発生時に
				データを送信してきたWebSocketクライアント接続(１つ)にのみ
				データを送信することができる。
																		"WS04B20D98A65F4F"

PayloadType		WebSocketフレームタイプを示す。"text" または "binary" が格納される。
																		"text"
PayloadSize		受信したフレームデータのサイズ(バイト数)が入る			"128"

PayloadData		受信したフレームのバイナリデータを16進数文字列に変換した
				ものが格納されますPayloadType が "text" の場合は、
				ペイロードデータに格納されたUTF-8 文字列コードのバイト列
				が格納されています。イベントハンドラ中でこれらの文字列
				データをデコードする処理がデフォルトで記述されていますので、
				UTF-8 文字列を扱う場合にはデコード後の変数を利用することができます。
																		"010203414243"

]]

------------------------------------------------------------------------------------------
-- 受信したPayloadType が "text" の場合には
-- バイナリデータ列を UTF-8 文字列としてデコードしたものを PayloadString 変数に格納する
------------------------------------------------------------------------------------------
local PayloadString = ""
if g_params["PayloadType"] == 'text' then
	local pub_len = tonumber(g_params["PayloadSize"])
	PayloadString = readUTF_hex(bit_tohex(pub_len,4) .. g_params["PayloadData"])
end

if PayloadString ~= "" then
	log_msg("/" .. g_params["Channel"] .. "/" .. g_params["SessionToken"] .. " [" .. g_params["WebSocketID"] .. "] " .. PayloadString,g_script)
else
	log_msg("/" .. g_params["Channel"] .. "/" .. g_params["SessionToken"] .. " [" .. g_params["WebSocketID"] .. "] " .. g_params["PayloadData"],g_script)
end

-----------------------------------------------------------------------------
-- 送信してきたメッセージ内容をそのままクライアント側に送り返す(Echo)例
-----------------------------------------------------------------------------
--[[
if PayloadString ~= "" then
	if not websocket_emit_text(g_params["WebSocketID"],PayloadString) then error() end
end
]]

-------------------------------------------------------------------------------------------------------------------------------------
-- 下記のサンプルスクリプトの説明は、ホームページの Blog記事 "WebSocket を使用した双方向通信リアルタイムアプリの作成" をご覧ください
-------------------------------------------------------------------------------------------------------------------------------------
--[[
if (g_params["Channel"] == "led_websock") and (PayloadString ~= "") then
	local msg
	local payload_tbl = g_json.decode(PayloadString)

	-- クライアント起動時に最新の LED(GPIO#18) ステータスをイベント送信元にだけ送信する
	if payload_tbl.event == "client_ready" then
		local stat,cur_val = raspi_gpio_read(18)
		if not stat then error() end
		if cur_val then
			msg = '{"led":"on"}'
		else
			msg = '{"led":"off"}'
		end
		websocket_emit_text(g_params["WebSocketID"],msg)
	end

	-- LED 操作ボタン(Webアプリ側のGUI)を押したときは LED(GPIO#18) の状態を反転させる
	-- RASPI_CHANGE_DETECT イベントハンドラ中のタクトスイッチ(GPIO#23)を押したときの処理と同じ内容
	if payload_tbl.event == "toggle_click" then
		local stat,cur_val = raspi_gpio_read(18)
		if not stat then error() end
		if not raspi_gpio_write(18,not cur_val) then error() end
	end

end
]]
