
--[[

●機能概要

XBee デバイスマスターを作成または更新する。
シリアルデバイスから "XBEE" デバイスタイプのシリアルポートに対して
xbee_rebuild_master() を実行して XBee デバイスリストを更新する。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				XBee デバイスをローカルに接続しているシリアルポート名またはタイトル名
				省略した場合には abs_agent に登録済みのシリアルデバイスを検索して
				デバイスタイプが "XBEE" のものを自動的に選択する。このとき、シリアル
				デバイスが見つからなかった場合や、2 つ以上の "XBEE" デバイスが見つかった
				場合にはエラーとなる。
																		"/dev/ttyUSB0"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"
com				ローカル XBee デバイスが接続されたシリアルポート名		"/dev/ttyUSB0"

●備考

●変更履歴

2019/6/17   初版作成

copyright(c) 2019 All Blue System

]]

local xbee_com_port

if g_params["com"] and (g_params["com"] ~= "") then
	xbee_com_port = g_params["com"]
else
	-- シリアルポートを検索して XBee デバイスタイプのポート名取得を試みる
	local xbee_port_count = 0
	local stat,com,title,dev_type,buff = serial_all_list()
	if not stat then error() end
	for key,val in ipairs(com) do
		if dev_type[key] == "XBEE" then
			xbee_port_count = xbee_port_count + 1
			xbee_com_port = val
		end
	end
	if xbee_port_count == 0 then
		script_result("Status","*ERROR* serial(type:XBEE) port not found.")
		return
	end
	if xbee_port_count > 1 then
		script_result("Status","*ERROR* multiple serial(type:XBEE) ports.")
		return
	end
end
script_result("com",xbee_com_port)

log_msg("rebuilding master.. xbee_com_port: " .. xbee_com_port,g_script)
local stat = xbee_rebuild_master(xbee_com_port)
if stat then
	script_result("Status","OK")
else
	script_result("Status","*ERROR* xbee_rebuild_master() failed, port = " .. xbee_com_port)
end
