
--[[

●機能概要

リモート XBeeデバイスに接続している、TDCP モニタプログラムが動作しているコントローラデバイスで TDCPコマンドを実行する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値	値				           								値の例
---------------------------------------------------------------------------------
com		XBee デバイスをローカルに接続しているシリアルポート名またはタイトル名
		省略した場合には abs_agent に登録済みのシリアルデバイスを検索して
		デバイスタイプが "XBEE" のものを自動的に選択する。このとき、シリアル
		デバイスが見つからなかった場合や、2 つ以上の "XBEE" デバイスが見つかった
		場合にはエラーとなる。
																"/dev/ttyUSB0"
tdcp_cmd	string
		コントローラデバイス側で実行する TDCP コマンド文字列
																"lcd_disp,1,Hello World!!"
dest
		送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス
																"Device2"

no_result
		リプライを受信しない場合にこのパラメータを指定する。(値は何でも構わない)
		パラメータ省略時は、コマンド実行結果を示すリプライデータを受信する。
																"1"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値	値		            									値の例
---------------------------------------------------------------------------------
Stauts	処理が成功した場合には "OK" が入る
		指定したサービスが提供されていない等で、サーバーから結果を正常に
		受け取れなかった場合にはエラーメッセージが入る
		リプライフレーム中のステータスバイトの値が 0x00 以外の場合にも
		エラーメッセージが格納される。
																"OK"

com		ローカル XBee デバイスが接続されたシリアルポート名		"/dev/ttyUSB0"

count	TDCP コントローラから送信されたリプライデータ中のTDCPカラム数(n)	"3"		
1			<tdcp_result#1>	(TDCP prefix "$$$xxxxx")
2			<tdcp_result#2>	(command result status "0": fail "1":success)
3			<tdcp_result#3>	(reply parameter#1)
4			<tdcp_result#4>	(reply parameter#2)
..
..
n			<tdcp_result#n> (reply parameter#n-2)


data	TDCP コントローラから送信されたリプライデータ文字列。
		no_result パラメータを指定した場合には data パラメータは返されない。

●備考

●変更履歴

2019/6/25	初版作成

abs_agent	copyright(c) 2019 All Blue System

]]

local tdcp_cmd
if g_params["tdcp_cmd"] and (g_params["tdcp_cmd"] ~= "") then
	tdcp_cmd = g_params["tdcp_cmd"]
else
	script_result("Status","*ERROR* tdcp_cmd parameter not found.")
	return
end

local no_result = false
if g_params["no_result"] then
	no_result = true
end

--------------------------------------------------------------------
-- ローカル XBee デバイスが接続されたシリアルポート名を取得する
--------------------------------------------------------------------
local com
if g_params["com"] and (g_params["com"] ~= "") then
	com = g_params["com"]
else
	-- シリアルポートを検索して XBee デバイスタイプのポート名取得を試みる
	local xbee_port_count = 0
	local stat,port,title,dev_type,buff = serial_all_list()
	if not stat then error() end
	for key,val in ipairs(port) do
		if dev_type[key] == "XBEE" then
			xbee_port_count = xbee_port_count + 1
			com = val
		end
	end
	if xbee_port_count == 0 then
		script_result("Status","*ERROR* serial(type:XBEE) port not found.")
		return
	end
	if xbee_port_count > 1 then
		script_result("Status","*ERROR* multiple serial(type:XBEE) ports.")
		return
	end
end
script_result("com",com)

--------------------------------------------------------------------
-- TDCP コマンド実行対象のデバイスを選択する
--------------------------------------------------------------------
local dest = ""
if g_params["dest"] and (g_params["dest"] ~= "") then
	dest = g_params["dest"]
else
	script_result("Status","*ERROR* dest parameter is null.")
	return
end

--------------------------------------------------------------------
-- TDCP コマンド実行とリプライ受信
--------------------------------------------------------------------
local stat,result = xbee_tdcp_command(com,dest,tdcp_cmd,no_result)
if not stat then
	script_result("Status","*ERROR* xbee_tdcp_command() failed")
	return
end
if result then
	for i,v in ipairs(result) do
		script_result(i,v)
	end
	script_result("count",tostring(#result))
else
	script_result("count","0")
end

script_result("Status","OK")
