
--[[
********************************************************************************************

	グローバル共有変数、グローバル共有文字列リストに数値データを保存するためのライブラリ
	このライブラリ関数を使用すると、Lua の暗黙的な数値型と文字列型の相互変換機能に頼らずに、
	数値型の値を直接保存することができる。

	このライブラリでは新しく xxxx_shared_num()、xxxx_shared_numlist() ライブラリ関数を定義
	している。これらのライブラリ関数のコール形式はそれぞれのライブラリ関数の元の文字列型を扱う
	xxxx_shared_data(), xxxx_shared_strlist() ライブラリ関数と同ー。ただし、value パラメータは
	Lua 数値型で、value_tbl パラメータはLua数値配列を扱うように変更している。

	内部では IEEE-754 倍精度浮動小数点データを16進数文字列に変換して、文字列エントリとして
	管理している。

	copyright(c) 2019 All Blue System 

********************************************************************************************
]]

-------------------------------------------------------------------
-- stat = set_shared_num(key, value [,no_global_watch])
-- stat = set_shared_num(value_tbl)
-------------------------------------------------------------------
function set_shared_num(...)
	local argv = {...}
	local stat
	local hexstr
	local hexstr_tbl

	if type(argv[1]) == "string" then
		if type(argv[2]) ~= "number" then return false end
		hexstr = num_to_hex(argv[2])
		argv[2] = hexstr
		return set_shared_data(table.unpack(argv))
	end

	if type(argv[1]) == "table" then
		hexstr_tbl = {}
		for k,v in pairs(argv[1]) do
			hexstr = num_to_hex(v)
			hexstr_tbl[k] = hexstr
		end
		argv[1] = hexstr_tbl
		return set_shared_data(table.unpack(argv))
	end
	
	return false	-- invalid parameter
end

-------------------------------------------------------------------
-- stat, value = get_shared_num(key [,delete_flag])
-- stat, value_tbl = get_shared_num(key_arr [,delete_flag])
-------------------------------------------------------------------
function get_shared_num(...)
	local argv = {...}
	local stat
	local hexstr
	local hexstr_tbl
	local num
	local num_tbl

	if type(argv[1]) == "string" then
		stat, hexstr = get_shared_data(table.unpack(argv))
		if not stat then return false end
		stat, num = hex_to_num(hexstr)
		if not stat then return false end
		return true,num
	end

	if type(argv[1]) == "table" then
		stat, hexstr_tbl = get_shared_data(table.unpack(argv))
		if not stat then return false end
		num_tbl = {}
		for k,v in pairs(hexstr_tbl) do
			stat, num = hex_to_num(v)
			if not stat then return false end
			num_tbl[k] = num
		end
		return true,num_tbl
	end

	return false	-- invalid parameter
end

-------------------------------------------------------------------
-- stat = add_shared_numlist(channel, value [,unique [,limit]])
-------------------------------------------------------------------
function add_shared_numlist(...)
	local argv = {...}
	local hexstr

	if type(argv[2]) ~= "number" then return false end
	hexstr = num_to_hex(argv[2])
	argv[2] = hexstr
	return add_shared_strlist(table.unpack(argv))
end

-------------------------------------------------------------------
-- stat, numlist = get_shared_numlist(channel [,delete])
-------------------------------------------------------------------
function get_shared_numlist(...)
	local stat
	local strlist
	local numlist
	local num

	stat, strlist = get_shared_strlist(...)
	if not stat then return false end
	numlist = {}
	for k,v in ipairs(strlist) do
		stat, num = hex_to_num(v)
		if not stat then return false end
		table.insert(numlist,num)
	end
	return true,numlist
end

-------------------------------------------------------------------
-- stat = clear_shared_numlist(channel)
-------------------------------------------------------------------
function clear_shared_numlist(...)
	return clear_shared_strlist(...)
end

-------------------------------------------------------------------
-- stat = remove_shared_numlist(channel, value)
-------------------------------------------------------------------
function remove_shared_numlist(...)
	local argv = {...}
	local hexstr

	if type(argv[2]) ~= "number" then return false end
	hexstr = num_to_hex(argv[2])
	argv[2] = hexstr
	return remove_shared_strlist(table.unpack(argv))
end

-------------------------------------------------------------------
-- stat, count = count_shared_numlist(channel)
-------------------------------------------------------------------
function count_shared_numlist(...)
	return count_shared_strlist(...)
end

-------------------------------------------------------------------
-- stat, value = shift_shared_numlist(channel [,peek_only])
-------------------------------------------------------------------
function shift_shared_numlist(...)
	local stat
	local hexstr
	local num

	stat, hexstr = shift_shared_strlist(...)
	if not stat then return false end
	stat, num = hex_to_num(hexstr)
	if not stat then return false end
	return true,num
end

-------------------------------------------------------------------
-- stat, value = pop_shared_numlist(channel [,peek_only])
-------------------------------------------------------------------
function pop_shared_numlist(...)
	local stat
	local hexstr
	local num

	stat, hexstr = pop_shared_strlist(...)
	if not stat then return false end
	stat, num = hex_to_num(hexstr)
	if not stat then return false end
	return true,num
end

-------------------------------------------------------------------
-- stat, channel_list = list_shared_numlist([channel_prefix])
-------------------------------------------------------------------
function list_shared_numlist(...)
	return list_shared_strlist(...)
end

-------------------------------------------------------------------
-- stat = find_shared_numlist(channel, value)
-------------------------------------------------------------------
function find_shared_numlist(...)
	local argv = {...}
	local hexstr

	if type(argv[2]) ~= "number" then return false end
	hexstr = num_to_hex(argv[2])
	argv[2] = hexstr
	return find_shared_strlist(table.unpack(argv))
end

