
----------------------------------------------------------------------------
-- AQM1248A (128x48 pixel) 秋月電子製 LCD ディスプレイ出力用ライブラリ関数
----------------------------------------------------------------------------

raspi_RS_pin = 25

function raspi_AQM1248A_cmd(cmd)
	raspi_gpio_write(raspi_RS_pin,false)
	if not raspi_spi_write_byte(cmd) then error() end
end

function raspi_AQM1248A_display()
	local stat,cmd_data = graphic_dump(1)	-- ST7565用のコントロールコマンドとピクセルデータを出力
	if not stat then error() end
	for key,val in ipairs(cmd_data) do
		if (key % 2) == 0 then
			raspi_gpio_write(raspi_RS_pin,true)	-- Display RAM transfer
		else
			raspi_gpio_write(raspi_RS_pin,false)	-- Instructions taransfer
		end
		if not raspi_spi_write(val) then error() end
	end
end

function raspi_AQM1248A_init()

	-- Graphic LCD module initialize
	if not graphic_set_size(128,48) then error() end
	if not graphic_clear() then error() end

	-- Raspberry Pi SPI#0 configulation
	if not raspi_spi_config(true,"mode0",25,"cs0","low") then error() end
	
	-- AQM1248A initialize 
	raspi_AQM1248A_cmd(0xAE)	-- LCD Display off
	raspi_AQM1248A_cmd(0xA0)	-- ADC select normal
	raspi_AQM1248A_cmd(0xC8)	-- common output mode reverse direction
	raspi_AQM1248A_cmd(0xA3)	-- LCD bias set 1/7

	raspi_AQM1248A_cmd(0x2C)	-- power control 1
	wait_time(2)
	raspi_AQM1248A_cmd(0x2E)	-- power control 2
	wait_time(2)
	raspi_AQM1248A_cmd(0x2F)	-- power control 3

	raspi_AQM1248A_cmd(0x23)	-- Vo voltage regulator internal resistor ratio set
	raspi_AQM1248A_cmd(0x81)	-- Electronic volume mode set
	raspi_AQM1248A_cmd(0x1C)	-- Electronic volume value set
	raspi_AQM1248A_cmd(0xA4)	-- display all point normal
	raspi_AQM1248A_cmd(0x40)	-- display start line = 0
	raspi_AQM1248A_cmd(0xA6)	-- display normal
	raspi_AQM1248A_cmd(0xAF)	-- LCD Display on

end

function raspi_AQM1248A_try_init() -- まだデバイスが初期化されていないときにのみ初期化を行う
	local stat,flag = get_shared_data("AQM1248A_INITIALIZED") -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		set_shared_data("AQM1248A_INITIALIZED","1")
		raspi_AQM1248A_init() -- LCD AQM1248A 初期化
	end
end

