
--------------------------------------------------------------------------------------------------
-- AQM1248A (128x48 pixel) 秋月電子製 LCD ディスプレイ出力用ライブラリ関数(graphic2 ライブラリ版)
--------------------------------------------------------------------------------------------------
AQM1248A_SPI_PORT = 0
AQM1248A_RS_PIN = 25
AQM1248A_CLOCK_DIV = 25 -- spi clock: 250MHz/25 = 10MHz clock
AQM1248A_WIDTH = 128
AQM1248A_HEIGHT = 48
AQM1248A_DISP_BUFF_CH = "AQM1248ADispBuff"
AQM1248A_PAGE_SIZE = AQM1248A_WIDTH * AQM1248A_HEIGHT / 8
AQM1248A_PAGE_MAX = 2	-- ライブラリ内で使用可能なページ(フレームバンク)数

function AQM1248A_cmd(cmd)
	stat_check(raspi_gpio_write(AQM1248A_RS_PIN,false))
	stat_check(raspi_spi_write_byte(AQM1248A_SPI_PORT,cmd))
end

--------------------------------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- reload_page 指定時は、表示後に reload_page に指定した内容で page パラメータで指定したメモリ値を上書きする
--------------------------------------------------------------------------------------------------------------
function AQM1248A_display(page,reload_page)
	local p = 0
	if page then p = page end
	if p >= AQM1248A_PAGE_MAX then error() end
	local y_max_idx = math.floor(AQM1248A_HEIGHT/8) - 1
	local offset = 0
	for y_idx = 0,y_max_idx,1 do
		stat_check(raspi_gpio_write(AQM1248A_RS_PIN,false))								-- Instructions taransfer
		stat_check(raspi_spi_write(AQM1248A_SPI_PORT,bit_tohex(0xb0 + y_idx,2) .. "1000"))	-- set page/column address
		stat_check(raspi_gpio_write(AQM1248A_RS_PIN,true))									-- Display RAM transfer
		stat_check(raspi_spi_write_shmem(AQM1248A_SPI_PORT,AQM1248A_DISP_BUFF_CH,p * AQM1248A_PAGE_SIZE + offset,AQM1248A_WIDTH)) -- transfer bitmap data
		offset = offset + AQM1248A_WIDTH
	end
	if reload_page then
		if reload_page >= AQM1248A_PAGE_MAX then error() end
		stat_check(shmem_move(AQM1248A_DISP_BUFF_CH,reload_page * AQM1248A_PAGE_SIZE,p * AQM1248A_PAGE_SIZE,AQM1248A_PAGE_SIZE))
	end
end

function AQM1248A_init()
	log_msg("initializing AQM1248A device, port = ".. tostring(AQM1248A_SPI_PORT),g_script)

	-- Raspberry Pi SPI#0 configulation
	stat_check(raspi_spi_config(true,"mode0",AQM1248A_CLOCK_DIV,"cs0","low"))
	
	-- AQM1248A initialize 
	AQM1248A_cmd(0xAE)	-- LCD Display off
	AQM1248A_cmd(0xA0)	-- ADC select normal
	AQM1248A_cmd(0xC8)	-- common output mode reverse direction
	AQM1248A_cmd(0xA3)	-- LCD bias set 1/7

	AQM1248A_cmd(0x2C)	-- power control 1
	wait_time(2)
	AQM1248A_cmd(0x2E)	-- power control 2
	wait_time(2)
	AQM1248A_cmd(0x2F)	-- power control 3

	AQM1248A_cmd(0x23)	-- Vo voltage regulator internal resistor ratio set
	AQM1248A_cmd(0x81)	-- Electronic volume mode set
	AQM1248A_cmd(0x1C)	-- Electronic volume value set
	AQM1248A_cmd(0xA4)	-- display all point normal
	AQM1248A_cmd(0x40)	-- display start line = 0
	AQM1248A_cmd(0xA6)	-- display normal
	AQM1248A_cmd(0xAF)	-- LCD Display on

	------------------------------
	-- ディスプレイバッファ初期化
	------------------------------
	graphic2_init(AQM1248A_DISP_BUFF_CH,AQM1248A_WIDTH,AQM1248A_HEIGHT,AQM1248A_PAGE_MAX,-1)
end

function AQM1248A_try_init() -- まだデバイスが初期化されていないときにのみ初期化を行う
	local name = "AQM1248A_" .. tostring(AQM1248A_SPI_PORT) .. "_INITIALIZED"
	local flag = stat_check(get_shared_data(name)) -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		stat_check(set_shared_data(name,"1"))
		AQM1248A_init()
	end
end
