
------------------------------------------------------------
-- OLED(128x64 pixel) SSD1306 コントローラ用ライブラリ関数
------------------------------------------------------------

OLED1306_ADDR = "3C"

function raspi_OLED1306_display(bus)
	if not bus then bus = 1 end
	local cmd_data = stat_check(graphic_dump(1))	-- ST7565/SSD1306用のコントロールコマンドとピクセルデータ出力
	for key,val in ipairs(cmd_data) do
		if (key % 2) == 0 then
			stat_check(raspi_i2c_write(bus,OLED1306_ADDR,"40" .. val)) -- Display RAM transfer
		else
			stat_check(raspi_i2c_write(bus,OLED1306_ADDR,"00" .. val)) -- Instructions taransfer
		end
	end
end

function raspi_OLED1306_init(bus)
	if not bus then bus = 1 end
	log_msg("initializing OLED1306 device, bus = ".. tostring(bus),g_script)

	-- graphic library initialize
	stat_check(graphic_set_size(128,64))
	stat_check(graphic_clear())

	-- Raspberry Pi I2C clock configulation
	stat_check(raspi_i2c_clock(bus,375)) -- 150MHz / 375 = 400KHz clock

	-- SSD1306 initialize
	local data = ""
	data = data .. "00"			-- start command stream
	data = data .. "AE"			-- display off	
	data = data .. "D580"		-- set display clock divide ratio/oscillator frequency to 0x80
	data = data .. "A83F"		-- set multiplex ratio  to 0x3F (64 - 1)
	data = data .. "D300"		-- set display offset to 0
	data = data .. "40"			-- set display start line 0
	data = data .. "8D14"		-- set enable charge pump regulator to 0x14
	data = data .. "2002"		-- set page addressing mode
	--data = data .. "A0"		-- set segment column 0 to SEG0
	data = data .. "A1"			-- set segment column 127 to SEG0
	--data = data .. "C0"		-- COM output scan from COM0
	data = data .. "C8"			-- COM output scan to COM0
	data = data .. "817F"		-- contrast to 0x7F
	data = data .. "A4"			-- disable entire display
	data = data .. "A6"			-- set normal display
	data = data .. "2E"			-- deactivate scroll 
	data = data .. "21007F"		-- column range 0..127 
	data = data .. "220007"		-- page range 0..7
	data = data .. "AF"			-- display on
	stat_check(raspi_i2c_write(bus,OLED1306_ADDR,data))
end

function raspi_OLED1306_try_init(bus) -- まだデバイスが初期化されていないときにのみ初期化を行う
	if not bus then bus = 1 end
	local name = "OLED1306_" .. tostring(bus) .. "_INITIALIZED"
	local flag = stat_check(get_shared_data(name)) -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		stat_check(set_shared_data(name,"1"))
		raspi_OLED1306_init() -- OLED(128x64 pixel) SSD1306初期化
	end
end
