----------------------------------------------------------------------------------
-- OLED(128x64 pixel) SSD1306 コントローラ用ライブラリ関数(graphic2 ライブラリ版)
----------------------------------------------------------------------------------
OLED1306_ADDR = "3C"
OLED1306_CLOCK_DIV = 375 -- i2c clock: 150MHz/375 = 400KHz clock
OLED1306_I2C_BUS = 1
OLED1306_WIDTH = 128
OLED1306_HEIGHT = 64
OLED1306_DISP_BUFF_CH = "OLD1306DispBuff"
OLED1306_PAGE_SIZE = OLED1306_WIDTH * OLED1306_HEIGHT / 8
OLED1306_PAGE_MAX = 2	-- ライブラリ内で使用可能なページ(フレームバンク)数

--------------------------------------------------------------------------------------------------------------
-- ディスプレイバッファ中の指定ページ page の内容を LCD RAM に書き込む
-- reload_page 指定時は、表示後に reload_page に指定した内容で page パラメータで指定したメモリ値を上書きする
--------------------------------------------------------------------------------------------------------------
function OLED1306_display(page,reload_page)
	local p = 0
	if page then p = page end
	if p >= OLED1306_PAGE_MAX then error() end
	local y_max_idx = math.floor(OLED1306_HEIGHT/8) - 1
	local offset = 0
	for y_idx = 0,y_max_idx,1 do
		stat_check(raspi_i2c_write(OLED1306_I2C_BUS,OLED1306_ADDR,"00" .. bit_tohex(0xb0 + y_idx,2) .. "1000")) -- set page/column address
		stat_check(raspi_i2c_write_shmem(OLED1306_I2C_BUS,OLED1306_ADDR,OLED1306_DISP_BUFF_CH,p * OLED1306_PAGE_SIZE + offset,OLED1306_WIDTH,"40")) -- transfer bitmap data
		offset = offset + OLED1306_WIDTH
	end
	if reload_page then
		if reload_page >= OLED1306_PAGE_MAX then error() end
		stat_check(shmem_move(OLED1306_DISP_BUFF_CH,reload_page * OLED1306_PAGE_SIZE,p * OLED1306_PAGE_SIZE,OLED1306_PAGE_SIZE))
	end
end

function OLED1306_init()
	log_msg("initializing OLED1306 device, bus = ".. tostring(OLED1306_I2C_BUS),g_script)

	-- Raspberry Pi I2C clock configulation
	stat_check(raspi_i2c_clock(OLED1306_I2C_BUS,OLED1306_CLOCK_DIV)) -- 150MHz / 375 = 400KHz clock

	-- SSD1306 initialize
	local data = ""
	data = data .. "00"			-- start command stream
	data = data .. "AE"			-- display off	
	data = data .. "D580"		-- set display clock divide ratio/oscillator frequency to 0x80
	data = data .. "A83F"		-- set multiplex ratio  to 0x3F (OLED1306_HEIGHT(64) - 1)
	data = data .. "D300"		-- set display offset to 0
	data = data .. "40"			-- set display start line 0
	data = data .. "8D14"		-- set enable charge pump regulator to 0x14
	data = data .. "2002"		-- set page addressing mode
	--data = data .. "A0"		-- set segment column 0 to SEG0
	data = data .. "A1"			-- set segment column 127 to SEG0
	--data = data .. "C0"		-- COM output scan from COM0
	data = data .. "C8"			-- COM output scan to COM0
	data = data .. "817F"		-- contrast to 0x7F
	data = data .. "A4"			-- disable entire display
	data = data .. "A6"			-- set normal display
	data = data .. "2E"			-- deactivate scroll 
	data = data .. "21007F"		-- column range 0..127 
	data = data .. "220007"		-- page range 0..7
	data = data .. "AF"			-- display on
	stat_check(raspi_i2c_write(OLED1306_I2C_BUS,OLED1306_ADDR,data))

	------------------------------
	-- ディスプレイバッファ初期化
	------------------------------
	graphic2_init(OLED1306_DISP_BUFF_CH,OLED1306_WIDTH,OLED1306_HEIGHT,OLED1306_PAGE_MAX,-1)
end

function OLED1306_try_init() -- まだデバイスが初期化されていないときにのみ初期化を行う
	local name = "OLED1306_" .. tostring(OLED1306_I2C_BUS) .. "_INITIALIZED"
	local flag = stat_check(get_shared_data(name)) -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		stat_check(set_shared_data(name,"1"))
		OLED1306_init() -- OLED(128x64 pixel) SSD1306初期化
	end
end
