
--[[

ScrollpHAT HD (17x7 pixel) PIMORONI LEDマトリックス出力用ライブラリ関数

このライブラリを使用する場合には、このファイルを格納しているディレクトリ名から
先頭についたアンダースコアを取り除いて "preload/100_RASPI/scrollphathd" に
リネームしてからabs_agent を再起動させてください。

ver1.3 scrollphathd_draw_jchar() 中の　fontx_8x8_get() からshmem_get_fontx() を使用するように変更

ver1.2 日本語フォント表示用の scrollphathd_draw_jstring(),scrollphathd_draw_jchar() 関数追加

ver1.1 scrollphathd_draw_char(),scrollphathd_draw_string() に bright パラメータ追加

ver1.0 initial revision

copyright(c) 2018 All Blue System 

]]

----------------------------------------------------------------------------
-- command register operation, set frame_page(0..0x7) or register_bank(0x0b)
----------------------------------------------------------------------------
function cmd_31FL3731(page)
	if not raspi_i2c_write(1,"74","FD" .. bit_tohex(page,2)) then error() end
end

-------------------------
-- AGC control register
-------------------------
function agc_ctrl_31FL3731(mode)
	cmd_31FL3731(0x0b) -- set function register bank
	if not raspi_i2c_write(1,"74","00" .. bit_tohex(mode,2)) then error() end
end

--------------------
-- set active frame
--------------------
function frame_31FL3731(frame)
	cmd_31FL3731(0x0b) -- set function register bank
	if not raspi_i2c_write(1,"74","01" .. bit_tohex(frame,2)) then error() end
end

---------------------
-- shutdown register
---------------------
function shutdown_31FL3731(flag)
	cmd_31FL3731(0x0b) -- set function register bank
	if not raspi_i2c_write(1,"74","0A" .. bit_tohex(flag,2)) then error() end
end

---------------------
-- audio_sync register
---------------------
function audio_sync_31FL3731(flag)
	cmd_31FL3731(0x0b) -- set function register bank
	if not raspi_i2c_write(1,"74","06" .. bit_tohex(flag,2)) then error() end
end

------------------------------------------------------------------
-- 7x17 (0 始まり)の座標を Lua 一次元配列のインデックス値に変換
------------------------------------------------------------------
function scrollphathd_scr_idx(x,y)
	return 1 + y + (x * 7)
end

--[[
--------------------------------------------------------------
Screen Buffer中のピクセル値配列からPWMレジスタ転送用のバッファ
構造に変換するときの対応表を作成する。

Frame毎の PWMレジスタ(0x24) 以降に書き込む配列構造は下記になって
います。

(Offset)                       1 1 1 1 1 1     1
	1 2 3 4 5 6 7      8     9 0 1 2 3 4 5     6    (column)
　　-----------------------------------------------------------
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 0
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 1
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 2
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 3
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 4
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 5
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 6
	pxpxpxpxpxpxpx  .. 00 .. pxpxpxpxpxpxpx .. 00     16 * 7
	pxpxpxpxpxpxpx  .. 000000000000000000             16 * 8

px 部分が LED マトリックスに配線されている
00 部分の LEDが接続されていない ScreenBuffer をレジスタ値に変換すると 1 にマップされる。
--------------------------------------------------------------
]]
scrollphathd_pwm_map = {}
for i = 1,119,1 do
	scrollphathd_pwm_map[i] = 1
end

scrollphathd_pwm_map[scrollphathd_scr_idx(0,0)] = 16 * 8 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(0,1)] = 16 * 8 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(0,2)] = 16 * 8 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(0,3)] = 16 * 8 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(0,4)] = 16 * 8 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(0,5)] = 16 * 8 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(0,6)] = 16 * 8 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(1,0)] = 16 * 7 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(1,1)] = 16 * 7 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(1,2)] = 16 * 7 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(1,3)] = 16 * 7 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(1,4)] = 16 * 7 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(1,5)] = 16 * 7 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(1,6)] = 16 * 7 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(2,0)] = 16 * 6 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(2,1)] = 16 * 6 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(2,2)] = 16 * 6 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(2,3)] = 16 * 6 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(2,4)] = 16 * 6 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(2,5)] = 16 * 6 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(2,6)] = 16 * 6 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(3,0)] = 16 * 5 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(3,1)] = 16 * 5 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(3,2)] = 16 * 5 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(3,3)] = 16 * 5 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(3,4)] = 16 * 5 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(3,5)] = 16 * 5 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(3,6)] = 16 * 5 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(4,0)] = 16 * 4 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(4,1)] = 16 * 4 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(4,2)] = 16 * 4 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(4,3)] = 16 * 4 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(4,4)] = 16 * 4 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(4,5)] = 16 * 4 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(4,6)] = 16 * 4 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(5,0)] = 16 * 3 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(5,1)] = 16 * 3 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(5,2)] = 16 * 3 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(5,3)] = 16 * 3 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(5,4)] = 16 * 3 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(5,5)] = 16 * 3 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(5,6)] = 16 * 3 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(6,0)] = 16 * 2 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(6,1)] = 16 * 2 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(6,2)] = 16 * 2 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(6,3)] = 16 * 2 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(6,4)] = 16 * 2 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(6,5)] = 16 * 2 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(6,6)] = 16 * 2 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(7,0)] = 16 * 1 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(7,1)] = 16 * 1 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(7,2)] = 16 * 1 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(7,3)] = 16 * 1 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(7,4)] = 16 * 1 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(7,5)] = 16 * 1 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(7,6)] = 16 * 1 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(8,0)] = 16 * 0 + 1
scrollphathd_pwm_map[scrollphathd_scr_idx(8,1)] = 16 * 0 + 2
scrollphathd_pwm_map[scrollphathd_scr_idx(8,2)] = 16 * 0 + 3
scrollphathd_pwm_map[scrollphathd_scr_idx(8,3)] = 16 * 0 + 4
scrollphathd_pwm_map[scrollphathd_scr_idx(8,4)] = 16 * 0 + 5
scrollphathd_pwm_map[scrollphathd_scr_idx(8,5)] = 16 * 0 + 6
scrollphathd_pwm_map[scrollphathd_scr_idx(8,6)] = 16 * 0 + 7

scrollphathd_pwm_map[scrollphathd_scr_idx(9,0)] = 16 * 0 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(9,1)] = 16 * 0 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(9,2)] = 16 * 0 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(9,3)] = 16 * 0 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(9,4)] = 16 * 0 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(9,5)] = 16 * 0 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(9,6)] = 16 * 0 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(10,0)] = 16 * 1 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(10,1)] = 16 * 1 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(10,2)] = 16 * 1 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(10,3)] = 16 * 1 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(10,4)] = 16 * 1 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(10,5)] = 16 * 1 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(10,6)] = 16 * 1 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(11,0)] = 16 * 2 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(11,1)] = 16 * 2 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(11,2)] = 16 * 2 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(11,3)] = 16 * 2 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(11,4)] = 16 * 2 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(11,5)] = 16 * 2 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(11,6)] = 16 * 2 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(12,0)] = 16 * 3 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(12,1)] = 16 * 3 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(12,2)] = 16 * 3 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(12,3)] = 16 * 3 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(12,4)] = 16 * 3 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(12,5)] = 16 * 3 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(12,6)] = 16 * 3 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(13,0)] = 16 * 4 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(13,1)] = 16 * 4 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(13,2)] = 16 * 4 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(13,3)] = 16 * 4 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(13,4)] = 16 * 4 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(13,5)] = 16 * 4 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(13,6)] = 16 * 4 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(14,0)] = 16 * 5 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(14,1)] = 16 * 5 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(14,2)] = 16 * 5 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(14,3)] = 16 * 5 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(14,4)] = 16 * 5 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(14,5)] = 16 * 5 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(14,6)] = 16 * 5 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(15,0)] = 16 * 6 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(15,1)] = 16 * 6 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(15,2)] = 16 * 6 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(15,3)] = 16 * 6 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(15,4)] = 16 * 6 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(15,5)] = 16 * 6 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(15,6)] = 16 * 6 + 9

scrollphathd_pwm_map[scrollphathd_scr_idx(16,0)] = 16 * 7 + 15
scrollphathd_pwm_map[scrollphathd_scr_idx(16,1)] = 16 * 7 + 14
scrollphathd_pwm_map[scrollphathd_scr_idx(16,2)] = 16 * 7 + 13
scrollphathd_pwm_map[scrollphathd_scr_idx(16,3)] = 16 * 7 + 12
scrollphathd_pwm_map[scrollphathd_scr_idx(16,4)] = 16 * 7 + 11
scrollphathd_pwm_map[scrollphathd_scr_idx(16,5)] = 16 * 7 + 10
scrollphathd_pwm_map[scrollphathd_scr_idx(16,6)] = 16 * 7 + 9

------------------------
-- ScrollpHATHD 初期化
------------------------
function scrollphathd_init()
	log_msg("ScrollpHATHD initializing..","SCROLLPHATHD")

	--[[

	DisplayBuffer
	
	LED マトリックスに表示するピクセルデータを格納したグローバル共有メモリエリア。
	
	DisplayBuffer のサイズはLED マトリックスのサイズよりもかなり大きくとられて、
	X 軸方向にスクロール表示できるようになっている。
	
	DisplayBuffer のY軸(縦方向)のサイズは LED マトリックスのサイズと等しいが、
	X軸(横方向) は数倍のサイズを有する。現在 は 1536 pixel
	
	DisplayBuffer のインデックス(offset) は 0 から始まるため、ScreenBuffer の
	Lua 配列のインデックスとは違う点に注意すること。また同様に、X, Y 方向の座標値も
	0 から始まる。
	
	
	Y 軸(0..6) 7 [pixel]
	^
	|
	+-+ 
	|6| 6 ..                                          118                        10751
	+-+
	|5| 5 ..                                          117                        10750
	+-+                                                          
	|4| 4 11                                          116                        10749
	+-+ 
	|3| 3 10 <- display buffer array index            115                        10748
	+-+ 
	|2| 2  9                                          114                        10747
	+-+
	|1| 1  8                                          113                        10746
	+-+
	|0| 0  7                                          112                        10745
	+-+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+              +----+
	  | 0| 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|15|16|17|18|19|..............|1535| -> X 座標 (0..1535) 1536 pixel
   	  +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+              +----+

	]]

	-- create DisplayBuffer
	if not shmem_resize("DispBuff_31FL3731",1536 * 7) then error() end
	if not shmem_fill("DispBuff_31FL3731",0x00) then error() end -- clear buffer

	-- software reset
	shutdown_31FL3731(0)
	wait_time(10)
	shutdown_31FL3731(1)
	
	-- set picture mode
	agc_ctrl_31FL3731(0x00)
	
	-- disable audio_sync
	audio_sync_31FL3731(0x00)

	local led_control_reg = {
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff,
		0xff,0xff}
	
	-- enable all LED control flags (frame#0..#6)
	for reg=0,6,1 do
		cmd_31FL3731(reg)
		if not raspi_i2c_write(1,"74","00" .. tbl_to_hex(led_control_reg)) then error() end
	end
	
	-- clear LED matrix
	cmd_31FL3731(0) -- set frame#0
	if not raspi_i2c_write(1,"74","24" .. tbl_to_hex(new_tbl(144,0x00))) then error() end
	frame_31FL3731(0) -- display frame#0

	-- FONTX ライブラリ初期化、日本語フォントファイルを共有メモリに保存
	fontx_try_init()

end

--------------------------------------------------------------
-- SCROLLPHATHD デバイスとディスプレイバッファ初期化
-- まだデバイスが初期化されていないときにのみ初期化を行う
--------------------------------------------------------------
function scrollphathd_try_init()
	local stat,flag = get_shared_data("SCROLLPHATHD_INITIALIZED") -- 厳密な排他は行わないが充分である
	if flag ~= "1" then 
		set_shared_data("SCROLLPHATHD_INITIALIZED","1")
		scrollphathd_init()
	end
end

---------------------------------------------------------------------------------------
-- ディスプレイバッファ全体に 0x0 を書き込む
---------------------------------------------------------------------------------------
function scrollphathd_clear()
	if not shmem_fill("DispBuff_31FL3731",0x00) then error() end
end

---------------------------------------------------------------------------------------
-- x,y 座標にピクセル値を書き込む
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
---------------------------------------------------------------------------------------
function scrollphathd_draw_pixel(idx_arr,data_arr,x,y,val)
	if (x < 0) or (y < 0) or (x > 1535) or (y > 6) or (val < 0) or (val > 255) then return end 
	table.insert(idx_arr,y + x * 7)
	table.insert(data_arr,val)
end

---------------------------------------------------------------------------------------
-- ローカルに保持している描画データをディスプレイバッファにまとめて書き込む
---------------------------------------------------------------------------------------
function scrollphathd_draw_commit(idx_arr,data_arr)
	if not shmem_set("DispBuff_31FL3731",idx_arr,data_arr) then error() end
end

--[[
---------------------------------------------------------------------------------------
ディスプレイバッファ中のデータを LED マトリックスデバイス中の指定したフレームに
転送する。フレームデータ(一画面分)をディスプレイバッファから切り出すための、
X 軸の開始座標を指定する。
---------------------------------------------------------------------------------------

ScreenBuffer

DisplayBuffer から、LED マトリックスに表示する部分をとりだした配列。
ScreenBuffer 配列は LED Matrix のサイズと同じ大きさになる。

IS31FL3731 デバイスのPWMレジスタに転送する前に一時的に作成される Lua 配列。

ScreenBuffer に取り出した画面表示分のピクセル配列はガンマ補正を掛けた後、
LED マトリックスの配線に合わせて pwm_buff配列に変換される。
その後、I2C インターフェイスでデバイスに転送される。

 Y 軸(0..6) 7 [pixel]
 ^
 |
+-+
|6| 7 ..                                          119 
+-+
|5| 6 ..                                          118
+-+
|4| 5 ..                                          117
+-+                                                        17 x 7 = 119 [pixel]    
|3| 4 11                                          116
+-+ 
|2| 3 10 <- screen buffer array index             115
+-+ 
|1| 2  9                                          114
+-+
|0| 1  8                                          114
+-+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
  | 0| 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|15|16| -> X 座標 (0..16) 17 pixel
  +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+

]]

function scrollphathd_transfer(frame,x)
	if (frame > 6) or (x > 1519) then error() end
	local stat,screen_buffer = shmem_copy("DispBuff_31FL3731",x * 7,119)
	if not stat then error() end
	local pwm_buff = new_tbl(144,0x00) -- PWM レジスタ転送用バッファ作成
	local tmp_gamma
	for idx = 1,119,1 do
		tmp_gamma = screen_buffer[idx] + 1
		if tmp_gamma > 255 then
			pwm_buff[scrollphathd_pwm_map[idx]] = 255
		else
			pwm_buff[scrollphathd_pwm_map[idx]] = scrollphathd_gamma_table[tmp_gamma]
		end
	end
	cmd_31FL3731(frame)
	if not raspi_i2c_write(1,"74","24" .. tbl_to_hex(pwm_buff)) then error() end
end

-------------------------------------------------------------
-- LED マトリックスに指定した転送済みフレームデータを表示する
-------------------------------------------------------------
function scrollphathd_display(frame)
	if (frame > 6) then error() end
	frame_31FL3731(frame)
end

---------------------------------------------------------------------------------------
-- (x,y) 座標に幅w 高さh の四角形を描く。描画するピクセル値は val に指定する。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
---------------------------------------------------------------------------------------
function scrollphathd_draw_rect(idx_arr,data_arr,x,y,w,h,val)
	for i = x,x + w - 1,1 do
		scrollphathd_draw_pixel(idx_arr,data_arr,i,y,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,i,y + h - 1,val)
	end
	for i = y,y + h - 1 do
		scrollphathd_draw_pixel(idx_arr,data_arr,x,i,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x + w - 1,i,val)
	end
end

--------------------------------------------------------------------------------------------
-- (x,y) 座標に幅w 高さh の四角形を描く。輪郭と内部を指定したピクセル値 valで塗りつぶす
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_fill_rect(idx_arr,data_arr,x,y,w,h,val)
	for i = x,x + w - 1,1 do
		for j = y,y + h - 1,1 do
			scrollphathd_draw_pixel(idx_arr,data_arr,i,j,val)
		end
	end
end

---------------------------------------------------------------------------------------
-- (x0,y0) と (x1,y1) 座標間の線分を描く。描画するピクセル値は val に指定する。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
---------------------------------------------------------------------------------------
function scrollphathd_draw_line(idx_arr,data_arr,x0,y0,x1,y1,val)
	local steep = math.abs(y1 - y0) > math.abs(x1 - x0)
	if steep then
		x0,y0 = y0,x0
		x1,y1 = y1,x1
	end
	if x0 > x1 then
		x0,x1 = x1,x0
		y0,y1 = y1,y0
	end
    local dx = x1 - x0
    local dy = math.abs(y1 - y0)
    local err = math.modf(dx / 2)
    if y0 < y1 then
    	ystep = 1
    else
    	ystep = -1
	end
	while x0 <= x1 do
    	if steep then
			scrollphathd_draw_pixel(idx_arr,data_arr,y0,x0,val)
    	else
			scrollphathd_draw_pixel(idx_arr,data_arr,x0,y0,val)
		end
    	err = err - dy
    	if err < 0 then
			y0 = y0 + ystep
			err = err + dx
    	end
		x0 = x0 + 1
	end
end

---------------------------------------------------------------------------------------
-- (x0,y0) 座標に半径 r の円を描く。描画するピクセル値は val に指定する。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
---------------------------------------------------------------------------------------
function scrollphathd_draw_circle(idx_arr,data_arr,x0,y0,r,val)
	if r < 0 then return end
	local f = 1 - r
	local ddf_x = 1
	local ddf_y = -2 * r
	local x = 0
	local y = r

	scrollphathd_draw_pixel(idx_arr,data_arr,x0,y0 + r,val)
	scrollphathd_draw_pixel(idx_arr,data_arr,x0,y0 - r,val)
	scrollphathd_draw_pixel(idx_arr,data_arr,x0 + r,y0,val)
	scrollphathd_draw_pixel(idx_arr,data_arr,x0 - r,y0,val)

	while x<y do
		if f >= 0 then
			y = y - 1
			ddf_y = ddf_y + 2
			f = f + ddf_y
		end
		x = x + 1
		ddf_x = ddf_x + 2
		f = f + ddf_x
  
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 + x,y0 + y,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 - x,y0 + y,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 + x,y0 - y,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 - x,y0 - y,val)

		scrollphathd_draw_pixel(idx_arr,data_arr,x0 + y,y0 + x,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 - y,y0 + x,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 + y,y0 - x,val)
		scrollphathd_draw_pixel(idx_arr,data_arr,x0 - y,y0 - x,val)
	end
end

--------------------------------------------------------------------------------------------
-- (x0,y0) 座標に半径 r の円を描く。輪郭と内部を指定したピクセル値 valで塗りつぶす
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_fill_circle(idx_arr,data_arr,x0,y0,r,val)
	if r < 0 then return end
	local f = 1 - r
	local ddf_x = 1
	local ddf_y = -2 * r
	local x = 0
	local y = r

	for i = (y0 - r),(y0 + r) do
		scrollphathd_draw_pixel(idx_arr,data_arr,x0,i,val)
	end
	while x<y do
		if f >= 0 then
			y = y - 1
			ddf_y = ddf_y + 2
			f = f + ddf_y
		end
		x = x + 1
		ddf_x = ddf_x + 2
		f = f + ddf_x
		for i = (y0 - y),(y0 + y) do
			scrollphathd_draw_pixel(idx_arr,data_arr,x0 + x,i,val)
			scrollphathd_draw_pixel(idx_arr,data_arr,x0 - x,i,val)
		end
		for i = (y0 - x),(y0 + x) do
			scrollphathd_draw_pixel(idx_arr,data_arr,x0 + y,i,val)
			scrollphathd_draw_pixel(idx_arr,data_arr,x0 - y,i,val)
		end
	end
end

-----------------------------------------------------------
-- フォントテーブル定義
-----------------------------------------------------------
scrollphathd_font_tbl ={scrollphathd_font_3x5,scrollphathd_font_5x7sm}

--------------------------------------------------------------------------------------------
-- (x0,y0) 座標に c (ASCIIコード) で指定した文字を描く。座標は文字の左下に合わせる。
-- 描画時は font (フォント番号)で定義されたフォントデータを使用する。
-- 描画時のピクセル値はフォントデータ中に設定された値を使用する。
-- bright パラメータを 0 .. 1 の範囲で小数値で指定すると、描画するときのフォントデータ値を
-- 調整することができる。bright パラメータを省略した場合には 1 を指定したのと同じ。
-- リターン値は w,h で、描画した文字のピクセル幅と高さデータが入る。
-- c で指定した文字フォントが無かった場合やその他のエラー発生時には w,h に 0,0 が返る。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_draw_char(idx_arr,data_arr,x0,y0,c,font,bright)
	local w,h = 0,0
	if font > #scrollphathd_font_tbl then return 0,0 end
	local fdata = scrollphathd_font_tbl[font][c]
	if not fdata then return 0,0 end
	w = fdata[1]
	h = fdata[2]
	if (w * h + 2) ~= #fdata then return 0,0 end -- フォントデータは正しいか?
	for i = 1,h do
		for j = 1,w do
			if bright then
				scrollphathd_draw_pixel(idx_arr,data_arr,x0 + j - 1,y0 + h - i,math.modf(fdata[2 + w * (i - 1) + j] * bright))
			else
				scrollphathd_draw_pixel(idx_arr,data_arr,x0 + j - 1,y0 + h - i,fdata[2 + w * (i - 1) + j])
			end
		end
	end
	return w,h
end

--------------------------------------------------------------------------------------------
-- (x0,y0) 座標に c で指定した(Shift-JISコード並びをBigEndian形式で１つの整数値にしたコード)
-- 文字を描く。座標は文字の左下に合わせる。
-- 描画時は fontx に指定したフォントテーブルを使用する。現在は 8x8全角フォントのみに対応。
-- 描画時のピクセル値は 0xaa を使用する。
-- bright パラメータを 0 .. 1 の範囲で小数値で指定すると、描画するときのフォントデータ値を
-- 調整することができる。bright パラメータを省略した場合には 1 を指定したのと同じ。
-- リターン値は w,h で、描画した文字のピクセル幅と高さデータが入る。
-- c で指定した文字フォントが無かった場合やその他のエラー発生時には w,h に 0,0 が返る。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_draw_jchar(idx_arr,data_arr,x0,y0,c,fontx,bright)
	local pwm
	local w,h = 0,0
	local stat,fdata = shmem_get_fontx(fontx,c)
	if not stat then return 0,0 end
	w = 7 -- フォント右端1ピクセル幅を破棄
	h = 7 -- フォント最下段の1ラインを破棄
	if #fdata < 7 then return 0,0 end -- フォントデータは正しいか?
	for i = 1,h do
		for j = 1,w do
			if bit_and(fdata[i],bit_lshift(0x01,8 - j)) > 0 then
				pwm = 0xaa
				if bright then
					pwm = math.modf(pwm * bright)
				end 
			else
				pwm = 0
			end
			scrollphathd_draw_pixel(idx_arr,data_arr,x0 + j - 1,y0 + h - i,pwm)
		end
	end
	return w,h
end

--------------------------------------------------------------------------------------------
-- (x0,y0) 座標に str (ASCII文字列) で指定した文字列を描く。座標は最初の文字の左下に合わせる。
-- 描画時は font (フォント番号)で定義されたフォントデータを使用する。
-- 描画時のピクセル値はフォントデータ中に設定された値を使用する。
-- bright パラメータを 0 .. 1 の範囲で小数値で指定すると、描画するときのフォントデータ値を
-- 調整することができる。bright パラメータを省略した場合には 1 を指定したのと同じ。
-- リターン値は str_w,max_h で、描画した文字列全体のピクセル幅と高さデータが入る。
-- 文字間は１ピクセル分の隙間をとる。
-- 文字フォントが存在しなかった場合には、その文字は描画されない。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_draw_string(idx_arr,data_arr,x0,y0,str,font,bright)
	local max_h = 0
	local str_w = 0
	local ch_arr = str_to_tbl(str)
	if #ch_arr == 0 then return 0,0 end
	local font_w,font_h
	for k,v in ipairs(ch_arr) do
		font_w,font_h = scrollphathd_draw_char(idx_arr,data_arr,x0,y0,v,font,bright)
		if font_w > 0 then
			str_w = str_w + font_w + 1
			x0 = x0 + font_w + 1
		end
		if font_h > max_h then max_h = font_h end
	end
	return str_w - 1,max_h
end

--------------------------------------------------------------------------------------------
-- (x0,y0) 座標に str (日本語やASCII) で指定した文字列を描く。座標は最初の文字の左下に合わせる。
-- 英数字を描画するときの font (フォント番号)は ２(scrollphathd_font_5x7sm)を使用する。
-- 英数字の描画時のピクセル値はフォントデータ中に設定された値を使用する。
-- 日本語を描画するときのフォントテーブルを fontx パラメータに指定する。
-- bright パラメータを 0 .. 1 の範囲で小数値で指定すると、描画するときのフォントデータ値を
-- 調整することができる。bright パラメータを省略した場合には 1 を指定したのと同じ。
-- リターン値は str_w,max_h で、描画した文字列全体のピクセル幅と高さデータが入る。
-- 文字間は１ピクセル分の隙間をとる。
-- 文字フォントが存在しなかった場合には、その文字は描画されない。
-- ディスプレイバッファへの書き込みは、複数の描画データをまとめて draw_commit() で行う
--------------------------------------------------------------------------------------------
function scrollphathd_draw_jstring(idx_arr,data_arr,x0,y0,str,fontx,bright)
	local max_h = 0
	local str_w = 0
	local cstat,ch_arr = str_to_tbl2(str,2) -- マルチバイト文字は Shift-JIS 並びの BigEndian に変換する
	if not cstat then
		ch_arr = str_to_tbl(str) -- 強制的に1バイトデータの表示に変更
	end
	if #ch_arr == 0 then return 0,0 end
	local font_w,font_h
	for k,v in ipairs(ch_arr) do
		if (v >= 0x20) and (v <= 0x7e) then 
			font_w,font_h = scrollphathd_draw_char(idx_arr,data_arr,x0,y0,v,2,bright) -- scrollphathd_font_5x7sm 使用
		else
			font_w,font_h = scrollphathd_draw_jchar(idx_arr,data_arr,x0,y0,v,fontx,bright)
		end
		if font_w > 0 then
			str_w = str_w + font_w + 1
			x0 = x0 + font_w + 1
		end
		if font_h > max_h then max_h = font_h end
	end
	return str_w - 1,max_h
end
