

--[[

フォントはテーブル変数で提供されていて、文字コードと同じインデックス値の要素に
フォントデータが配列で格納されている。

フォントデータは描画時のピクセル値(PWM)を並べたもので、フォントの左上から右下に向かって
２次元データを１次元の配列に並べている。

各文字フォントのインデックス1,2 には、各文字ごとのフォントデータの幅と高さのピクセル値が
格納されている。

フォントテーブルを新たに定義した場合には SCROLLPHATHD.lua ファイル中で定義されている、
scrollphathd_font_tbl[] テーブルにフォントテーブルを追加すること。

このライブラリを使用する場合には、このファイルを格納しているディレクトリ名から
先頭についたアンダースコアを取り除いて "preload/100_RASPI/scrollphathd" に
リネームしてからabs_agent を再起動させてください。

2018/04/04 abs_agent オールブルーシステム 

******

フォントデータのオリジナルは Pimoroni社の Scroll pHAT HD ライブラリです。
この中のフォントデータを元に Lua テーブル形式に変換したものを同梱しています。
オリジナルソースファイルは下記の URL で公開されています。

https://github.com/pimoroni/scroll-phat-hd

また、Pimoroni社の Scroll pHAT HD ライブラリのライセンス条項を下記に示します。

MIT License

Copyright (c) 2017 Pimoroni Ltd.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

]]


------------
-- 3x5 font
------------
scrollphathd_font_3x5 = {}

--[[
scrollphathd_font_3x5[0x20] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00}
]]

scrollphathd_font_3x5[0x20] = {2,5,
0x00,0x00,
0x00,0x00,
0x00,0x00,
0x00,0x00,
0x00,0x00}

scrollphathd_font_3x5[0x21] = {1,5,
0xff,
0xff,
0xff,
0x00,
0xff}

scrollphathd_font_3x5[0x22] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x23] = {3,5,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x24] = {3,5,
0x00,0xff,0xff,
0xff,0xff,0x00,
0x00,0xff,0xff,
0xff,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x25] = {3,5,
0xff,0x00,0x00,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00,
0x00,0x00,0xff}

scrollphathd_font_3x5[0x26] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x27] = {1,5,
0xff,
0xff,
0x00,
0x00,
0x00}

scrollphathd_font_3x5[0x28] = {2,5,
0x00,0xff,
0xff,0x00,
0xff,0x00,
0xff,0x00,
0x00,0xff}

scrollphathd_font_3x5[0x29] = {2,5,
0xff,0x00,
0x00,0xff,
0x00,0xff,
0x00,0xff,
0xff,0x00}

scrollphathd_font_3x5[0x2a] = {3,5,
0xff,0x00,0xff,
0x00,0xff,0x00,
0xff,0xff,0xff,
0x00,0xff,0x00,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x2b] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0x00,
0xff,0xff,0xff,
0x00,0x00,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x2c] = {2,5,
0x00,0x00,
0x00,0x00,
0x00,0x00,
0x00,0xff,
0xff,0x00}

scrollphathd_font_3x5[0x2d] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0xff,0xff,
0x00,0x00,0x00,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x2e] = {1,5,
0x00,
0x00,
0x00,
0xff,
0xff}

scrollphathd_font_3x5[0x2f] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x30] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x31] = {3,5,
0x00,0xff,0x00,
0xff,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x32] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x33] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x34] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff}

scrollphathd_font_3x5[0x35] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x36] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x37] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff}

scrollphathd_font_3x5[0x38] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x39] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x3a] = {1,5,
0x00,
0xff,
0x00,
0xff,
0x00}

scrollphathd_font_3x5[0x3b] = {2,5,
0x00,0x00,
0x00,0xff,
0x00,0x00,
0x00,0xff,
0xff,0x00}

scrollphathd_font_3x5[0x3c] = {3,5,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00,
0x00,0xff,0x00,
0x00,0x00,0xff}

scrollphathd_font_3x5[0x3d] = {3,5,
0x00,0x00,0x00,
0xff,0xff,0xff,
0x00,0x00,0x00,
0xff,0xff,0xff,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x3e] = {3,5,
0xff,0x00,0x00,
0x00,0xff,0x00,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00}

scrollphathd_font_3x5[0x3f] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0xff,0x00,
0x00,0x00,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x40] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0x00,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x41] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x42] = {3,5,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x43] = {3,5,
0x00,0xff,0xff,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0x00,0x00,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x44] = {3,5,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x45] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x46] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0x00,0x00}

scrollphathd_font_3x5[0x47] = {3,5,
0x00,0xff,0xff,
0xff,0x00,0x00,
0xff,0xff,0xff,
0xff,0x00,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x48] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x49] = {3,5,
0xff,0xff,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x4a] = {3,5,
0x00,0x00,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x4b] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x4c] = {3,5,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x4d] = {3,5,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x4e] = {3,5,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x4f] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x50] = {3,5,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00,
0xff,0x00,0x00,
0xff,0x00,0x00}

scrollphathd_font_3x5[0x51] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x52] = {3,5,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0xff,0x00,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x53] = {3,5,
0x00,0xff,0xff,
0xff,0x00,0x00,
0x00,0xff,0x00,
0x00,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x54] = {3,5,
0xff,0xff,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x55] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x56] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x57] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x58] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x59] = {3,5,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x5a] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x5b] = {3,5,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x5c] = {3,5,
0x00,0x00,0x00,
0xff,0x00,0x00,
0x00,0xff,0x00,
0x00,0x00,0xff,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x5d] = {3,5,
0xff,0xff,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff,
0x00,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x5e] = {3,5,
0x00,0xff,0x00,
0xff,0x00,0xff,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x5f] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x60] = {2,5,
0xff,0x00,
0x00,0xff,
0x00,0x00,
0x00,0x00,
0x00,0x00}

scrollphathd_font_3x5[0x61] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x62] = {3,5,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x63] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0xff,0xff,
0xff,0x00,0x00,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x64] = {3,5,
0x00,0x00,0xff,
0x00,0x00,0xff,
0x00,0xff,0xff,
0xff,0x00,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x65] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0xff,
0xff,0x00,0xff,
0xff,0xff,0x00,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x66] = {3,5,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0xff,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x67] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0xff,
0xff,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x68] = {3,5,
0xff,0x00,0x00,
0xff,0x00,0x00,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x69] = {1,5,
0xff,
0x00,
0xff,
0xff,
0xff}

scrollphathd_font_3x5[0x6a] = {3,5,
0x00,0x00,0xff,
0x00,0x00,0x00,
0x00,0x00,0xff,
0x00,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x6b] = {3,5,
0x00,0x00,0x00,
0xff,0x00,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x6c] = {3,5,
0xff,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x6d] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x6e] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0xff,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x6f] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0xff,0x00,
0xff,0x00,0xff,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x70] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0x00,
0xff,0x00,0xff,
0xff,0xff,0x00,
0xff,0x00,0x00}

scrollphathd_font_3x5[0x71] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0x00,
0xff,0x00,0xff,
0x00,0xff,0xff,
0x00,0x00,0xff}

scrollphathd_font_3x5[0x72] = {3,5,
0x00,0x00,0x00,
0xff,0x00,0x00,
0xff,0xff,0xff,
0xff,0x00,0x00,
0xff,0x00,0x00}

scrollphathd_font_3x5[0x73] = {3,5,
0x00,0x00,0x00,
0x00,0xff,0xff,
0xff,0xff,0x00,
0x00,0xff,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x74] = {3,5,
0x00,0xff,0x00,
0xff,0xff,0xff,
0x00,0xff,0x00,
0x00,0xff,0x00,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x75] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0x00,0xff,
0xff,0x00,0xff,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x76] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0x00,0xff,
0xff,0xff,0xff,
0x00,0xff,0x00}

scrollphathd_font_3x5[0x77] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0x00,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x78] = {3,5,
0x00,0x00,0x00,
0x00,0x00,0x00,
0xff,0x00,0xff,
0x00,0xff,0x00,
0xff,0x00,0xff}

scrollphathd_font_3x5[0x79] = {3,5,
0x00,0x00,0x00,
0xff,0x00,0xff,
0x00,0xff,0xff,
0x00,0x00,0xff,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x7a] = {3,5,
0x00,0x00,0x00,
0xff,0xff,0xff,
0x00,0xff,0xff,
0xff,0xff,0x00,
0xff,0xff,0xff}

scrollphathd_font_3x5[0x7b] = {3,5,
0x00,0xff,0xff,
0x00,0xff,0x00,
0xff,0x00,0x00,
0x00,0xff,0x00,
0x00,0xff,0xff}

scrollphathd_font_3x5[0x7c] = {1,5,
0xff,
0xff,
0x00,
0xff,
0xff}

scrollphathd_font_3x5[0x7d] = {3,5,
0xff,0xff,0x00,
0x00,0xff,0x00,
0x00,0x00,0xff,
0x00,0xff,0x00,
0xff,0xff,0x00}

scrollphathd_font_3x5[0x7e] = {3,5,
0x00,0xff,0xff,
0xff,0xff,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00,
0x00,0x00,0x00}

scrollphathd_font_3x5[0x7f] = {3,5,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff,
0xff,0xff,0xff}


