
--[[
****************************************************************************************************

●機能概要

MGCP デバイスの GPIO操作用の API ライブラリ

copyright(c) 2019 All Blue System

****************************************************************************************************
]]



--[[
****************************************************************************************************

MGCP デバイスの GPIO ポート値を取得する

パラメータ
		node:string		MGCP ノード名
		gpio:number		GPIO ポート番号 0 .. 39

リターン値
		stat:boolean	true 成功、false 失敗
		val:number		GPIO 値(0 または 1)
						コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]
function mgcp_gpio_get(node,gpio)
	local stat,reply = mgcp_command(node,'{"command":"gpio_get","gpio":' .. tostring(gpio) .. '}',false,1000)
	if stat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.level
		else
			return false
		end
	else
		return false
	end

end

--[[
****************************************************************************************************

MGCP デバイスの GPIO ポート値を設定する

パラメータ
		node:string		MGCP ノード名
		gpio:number		GPIO ポート番号 0 .. 39
		val:number		GPIO 値(0 または 1)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_gpio_set(node,gpio,val)
	return mgcp_command(node,'{"command":"gpio_set","gpio":' .. tostring(gpio) .. ',"level":' .. tostring(val) .. '}',false,1000)
end

--[[
****************************************************************************************************

MGCP デバイスの GPIOポートを High - Low 繰り返し出力状態にする。

パラメータ
		node:string		MGCP ノード名
		gpio:number		GPIO ポート番号 0 .. 39
		flag:number		繰り返し設定フラグ("on" または "off")
		val:number		"off" 設定時の GPIO 値(0 または 1)
						オプションパラメータ

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]
function mgcp_gpio_blink(node,gpio,flag,val)
	if val == nil then
		return mgcp_command(node,'{"command":"gpio_blink","gpio":' .. tostring(gpio) .. ',"flag":"' .. flag .. '"}',false,1000)
	else
		return mgcp_command(node,'{"command":"gpio_blink","gpio":' .. tostring(gpio) .. ',"flag":"' .. flag .. '","level":' .. tostring(val) .. '}',false,1000)
	end
end
