
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する SPI デバイス操作用の API ライブラリ

copyright(c) 2019 All Blue System

****************************************************************************************************
]]

--[[
****************************************************************************************************

MGCP ノードに接続した SPI デバイスに複数のデータ列を書き込む

パラメータ
		node:string		MGCP ノード名
		port:number		SPI ポート番号 0 .. 1
		delay:number	ブロック書き込みするときの各書き込みトランザクション間のディレイ時間(ms)
		dc_gpio:number	データ・レジスタ切り替えライン出力用の GPIO ポート番号。-1 を指定するとこの機能を使用しない
		level:number	データブロックを書き込む前に dc_gpio に最初に出力する GPIO 値。0 または 1 を指定
		hexstr_csv:string	書き込むデータ(16進数文字列、1バイトあたり2桁)をCSV形式で並べた文字列
							例： '"12","AB","001234"'   各ブロックの16進数データはダブルコートで囲む事

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function mgcp_spi_write_block(node,port,delay,dc_gpio,level,hexstr_csv)
	return mgcp_command(node,'{"command":"spi_write_block","port":' .. tostring(port).. ',"delay":' .. tostring(delay).. 
											',"dc_gpio":' .. dc_gpio .. ',"level":' .. level .. ',"data":[' .. hexstr_csv .. ']}',false,3000)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込む
		
パラメータ
		node:string		MGCP ノード名
		port:number		SPI ポート番号 0 .. 1
		hexstr:string	書き込むデータ(16進数文字列、1バイトあたり2桁)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function mgcp_spi_write(node,port,hexstr)
	return mgcp_command(node,'{"command":"spi_write","port":' .. tostring(port).. ',"data":"' .. hexstr .. '"}',false,3000)
end

--[[
****************************************************************************************************

MGCP ノードに接続した SPI デバイスにデータを書き込みと同時にデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		SPIポート番号 0 .. 1
		hexstr:string	書き込むデータ(16進数文字列、1バイトあたり2桁)

リターン値
		stat:boolean	true 成功、false 失敗
		hexstr_rpl:string	SPIデバイスから読み込んだデータ(16進数文字列)。
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function mgcp_spi_write_read(node,port,hexstr)
	local stat,reply = mgcp_command(node,'{"command":"spi_write","port":' .. tostring(port).. ',"read_flag":1,"data":"' .. hexstr .. '"}',false,3000)
	if stat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.data
		else
			return false
		end
	else
		return false
	end
end

--[[
****************************************************************************************************

MGCP ノードに接続した SPI デバイスからデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		SPIポート番号 0 .. 1
		len:number		読み込むデータバイト数(最大 1500)

リターン値
		stat:boolean	true 成功、false 失敗
		hexstr_rpl:string	SPIデバイスから読み込んだデータ(16進数文字列)。
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function mgcp_spi_read(node,port,len)
	local stat,reply = mgcp_command(node,'{"command":"spi_read","port":' .. tostring(port).. ',"read_len":' .. tostring(len) .. '}',false,3000)
	if stat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.data
		else
			return false
		end
	else
		return false
	end
end

