
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する SPI デバイス操作用の API ライブラリ
ダイレクトコマンドモードを使用して、MGCP コマンドに比べて高速に動作する。

2020/09/06 ver1.0 新規作成

copyright(c) 2020 All Blue System

****************************************************************************************************
]]

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込む
		
パラメータ
		node:string				MGCP ノード名
		port:number				SPI ポート番号 0 .. 1
		data:table of number	書き込むき込むデータ

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function build_frame_spi_write(arr,port,data)
	table.insert(arr,0x20)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_byte(arr,0)
	for i,v in ipairs(data) do
		table.insert(arr,v)
	end
end

function dxcmd_spi_write(node,port,data)
	local frame = {}
	build_frame_spi_write(frame,port,data)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP ノードに接続した SPI デバイスにデータを書き込みと同時にデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		SPIポート番号 0 .. 1
		data:table of number	書き込むデータ

リターン値
		stat:boolean				true 成功、false 失敗
		rdata:table of number		SPIデバイスから読み込んだデータ
									コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function build_frame_spi_write_read(arr,port,data)
	table.insert(arr,0x20)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_byte(arr,1)
	for i,v in ipairs(data) do
		table.insert(arr,v)
	end
end

function dxcmd_spi_write_read(node,port,data)
	local frame = {}
	build_frame_spi_write_read(frame,port,data)
	return dxcmd_exec_reply_get(node,frame)
end

--[[
****************************************************************************************************

MGCP ノードに接続した SPI デバイスからデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		SPIポート番号 0 .. 1
		len:number		読み込むデータバイト数(最大 1023)

リターン値
		stat:boolean	true 成功、false 失敗
		rdata:table of number	SPIデバイスから読み込んだデータ
								コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function build_frame_spi_read(arr,port,len)
	table.insert(arr,0x21)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_word(arr,len)
end

function dxcmd_spi_read(node,port,len)
	local frame = {}
	build_frame_spi_read(frame,port,len)
	return dxcmd_exec_reply_get(node,frame)
end
