
//////////////////////////////////////
// WebSocketClient
//////////////////////////////////////
var ws_channel = "led_websock";
var ws_port = 9090;
var ws;

// Web(HTTP)サーバーと同じコンピュータで動作している WebSocketサーバーに接続
function connect_websocket_server(){
	ws = new WebSocket("ws://" + location.hostname + ":" + ws_port + "/" + ws_channel + "/" + session_token);
	
	ws.onopen = function (e) {
		// WebSocketサーバーへ接続したら、新しいクライアントが接続したことを知らせるイベントを送信する。
		// するとサーバーから最新の LED ステータスが送信されて、LEDチェックボックス GUI の状態と一致させることができる。
		ws.send('{"event":"client_ready"}');      
	}
			
	ws.onclose = function (e) {      
		log("Disconnected: " + e.reason);  
	}
			
	ws.onerror = function (e) {     
		$("body").pagecontainer("change","#websock_quit_dialog", { transition: "pop",role:"dialog"});
	}
	
	// WebSocketデータフレームを受信した
	ws.onmessage = function (e) {      
		if (typeof e.data == "string"){
			// JSON 文字列 {"led":"on"} または {"led":"off"} に従って LEDチェックボックスの状態を更新する。
			var obj = JSON.parse(e.data);
			if (obj.led == "on") {
				$("#LED_status_rdo").prop('checked',true).checkboxradio('refresh');
			}
			if (obj.led == "off") {
				$("#LED_status_rdo").prop('checked',false).checkboxradio('refresh');
			}
		}
	}
}

// WebSocket サーバー切断
function disconnect_websocket_server(){
	ws.close();
}

// WebSocketエラーのダイアログから復帰する場合はログイン画面に戻る
$( "#websock_error_ok_btn" ).on( "click", function(event, ui){
	session_token = ""; // abs_agent 側に残ったログインセッションは後で自動削除される
	$("body").pagecontainer("change","#login", { transition: "pop" });
});

//////////////////////////////////////
// LED 画面
//////////////////////////////////////

// LED操作ボタンが操作された
$("#led_toggle_btn").on( "click",function(event, ui){
	// イベントメッセージを送信することで、サーバー側の WEBSOCKET_DATAイベントハンドラ中で GPIO を操作して
	// LED の状態を反転させる。
	// LED(GPIO) の状態が変化すると、サーバー側でGPIO 値変化を検出して RASPI_CHANGE_DETECTイベントハンドラが起動する。
	// RASPI_CHANGE_DETECTイベントハンドラ中では、最新の LED(GPIO) ステータスを格納したメッセージを WebSocket 経由で
	// 全クライアントに送信する。
	ws.send('{"event":"toggle_click"}');      
});

// ページが表示された
$(document).on("pageshow","#main_page",function(event){
	log("main page");
});

//////////////////////////////////////
// login page
//////////////////////////////////////
 
// サーバー側のログイン処理が成功したら、WebSocketサーバーに接続を試みてメイン画面に移動する
function login_callback(data){
	if (data.Result == "Success"){
		session_token = data.SessionToken;

		connect_websocket_server();

		$("body").pagecontainer("change","#main_page", { transition: "none" });
	} else {
		$("body").pagecontainer("change","#login_error_dialog", { transition: "pop",role:"dialog" });
	} 
}

// サーバー側のログアウト処理が完了したらログイン画面に戻る
function logout_callback(data){
	session_token = "";
	$("#login_password").val("");

	$("body").pagecontainer("change","#login", { transition: "pop" });
}

// ログインボタンを押した
$( "#login_btn" ).on( "click", function(event, ui){
	var user = $("#login_name").val();
	var pass = $("#login_password").val();
	login(user,pass,"login_callback");
});

// ログアウトボタンを押した
$( "#logout_ok_btn" ).on( "click", function(event, ui){
	disconnect_websocket_server();
	logout("logout_callback");
});

// サーバーエラーのダイアログから復帰する場合はログイン画面に戻る
$( "#server_error_ok_btn" ).on( "click", function(event, ui){
	session_token = "";
	$("body").pagecontainer("change","#login", { transition: "pop" });
});

// ログインページが表示された
$(document).on("pageshow","#login",function(event){
	// 暗黙のセッショントークンが指定されている場合にはユーザー認証を省略する
	if (session_token != ""){
		connect_websocket_server();
		$("body").pagecontainer("change","#main_page", { transition: "pop" });
	}
});
